Package: CCAFE
Type: Package
Title: Case Control Allele Frequency Estimation
Version: 1.4.0
Authors@R: c(person("Hayley", "Wolff", email = "hayley.wolff@cuanschutz.edu", role = c("cre", "aut")))
Description: Functions to reconstruct case and control AFs from summary statistics. 
    One function uses OR, NCase, NControl, and SE(log(OR)). 
    The second function uses OR, NCase, NControl, and AF for the whole sample. 
License: GPL-3
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
Imports: dplyr, VariantAnnotation
Depends: R (>= 4.4.0)
Suggests: testthat (>= 3.0.0), rmarkdown, markdown, knitr, tidyverse,
        DescTools, cowplot, BiocStyle, GenomicRanges,
        SummarizedExperiment, S4Vectors, IRanges
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/wolffha/CCAFE/
BugReports: https://github.com/wolffha/CCAFE/issues
biocViews: GenomeWideAssociation, ComparativeGenomics, Genetics,
        Preprocessing, SNP, Software, WholeGenome
git_url: https://git.bioconductor.org/packages/CCAFE
git_branch: RELEASE_3_23
git_last_commit: 91b20f3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:37:38 UTC; biocbuild
Author: Hayley Wolff [cre, aut]
Maintainer: Hayley Wolff <hayley.wolff@cuanschutz.edu>
Built: R 4.6.0; ; 2026-04-29 04:36:12 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CCAFE_convertVCF)
export(CaseControl_AF)
export(CaseControl_SE)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)

# CCAFE
**C**ase **C**ontrol **A**llele **F**requency (AF) **E**stimation R Package

This repository contains the source code for the CaseControlAF R package which can be used to reconstruct the allele frequency (AF) for cases and controls separately given commonly available summary statistics. 

The package contains two functions:

1) CaseControl_AF
2) CaseControl_SE

See full documentation, vignettes, and examples here: (https://wolffha.github.io/CCAFE_documentation/)

## Download the package

To install this package using BioConductor (Not yet available):

```R
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("CCAFE")
```

To download this package using *devtools* in R:

```R
require(devtools)
devtools::install_github("https://github.com/wolffha/CCAFE/")
```

## CaseControl_AF

Use this function when you have the following statistics (for each variant)

* Number of cases
* Number of controls
* Odds Ratio (OR) or beta coefficient
* **AF** (allele frequency) for the total sample (cases and controls combined)

### Usage
**data**: a dataframe with a row for each variant and columns for OR and total AF

**N_case**: an integer for the number of case samples

**N_control**: an integer for the number of control samples

**OR_colname**: a string containing the exact column name in 'data' with the OR

**AF_total_colname**: a string containing the exact column name in 'data' with the total AF

Returns a dataframe with two columns: AF_case and AF_control. The number of rows is equal to the number of variants.

## CaseControl_SE
Use this function when you have the following statistics (for each variant)

* Number of cases
* Number of controls
* Odds Ratio (OR) or beta coefficient
* **SE** of the log(OR) for each variant

*Code adapted from ReACt GroupFreq function available here: (https://github.com/Paschou-Lab/ReAct/blob/main/GrpPRS_src/CountConstruct.c)*

### Usage
**data**: a dataframe where each row is a variant and columns for the OR, SE, chromosome, and position

**N_case**: an integer for the number of case samples

**N_control**: an integer for the number of control samples

**OR_colname**: a string containing the exact column name in *data* with the odds ratios

**SE_colname**: a string containing the exact column name in *data* with the standard errors

**position_colname**: a string containing the exact column name in *data* with the positions of the variants

**chromosome_colname**: a string containing the exact column name in *data* with the chromosome of the variants. 
Note, sex chromosomes can be either characters ('X', 'x', 'Y', 'y') or numeric where X=23 and Y=24

**sex_chromosomes**: boolean, TRUE if variants from sex chromosome(s) are included in the dataset

**do_correction**: boolean, TRUE if data is provided to correct the estimates using proxy MAFs

**remove_sex_chromosomes**: boolean, TRUE if variants on sex chromosomes should be removed. This is only necessary if *sex_chromosomes* == TRUE and the number of XX/XY individuals per case and control sample is NOT known

CaseControl_SE has the following optional inputs: 

If *sex_chromosomes* == TRUE and *remove_sex_chromosomes* == FALSE, then the following inputs are required:

**N_XX_case**: the number of XX chromosome case individuals

**N_XX_control**: the number of XX chromosome control individuals

**N_XY_case**: the number of XY chromosome case individuals

**N_XY_control**: the number of XY chromosome control individuals

If *do_correction* == TRUE, then data must be provided that includes harmonized data with proxy MAFs

**correction_data**: a dataframe with the following EXACT column names: CHR, POS, proxy_MAF, containing data for variants harmonized between the observed and proxy datasets

Returns the *data* dataframe with three additional columns with names: MAF_case, MAF_control and MAF_total containing the estimated minor allele frequency in the cases, controls, and total sample. The number of rows is equal to the number of variants. If proxyMAFs_colname is not NA, will include three additional columns containing the adjusted estimated MAFs (MAF_case_adj, MAF_control_adj, MAF_total_adj)

**NOTE:** This method assumes we are estimating the minor allele frequency (MAF)

### Examples and documentation

See full documentation, vignettes, and examples here: (https://wolffha.github.io/CCAFE_documentation/)

#' @title CaseControl_AF
#' @description
#' This is a function to derive the case and control AFs from GWAS summary
#' statistics when the user has access to the whole sample AF, the sample sizes,
#' and the OR (or beta).
#' If user has SE instead of sample AF use [CCAFE::CaseControl_SE()]
#'
#' @param data dataframe with each row being a variant and columns for
#' AF_total and OR
#' @param N_case the number of cases in the sample
#' @param N_control the number of controls in the sample
#' @param OR_colname a string containing the exact column name in 'data'
#' with the OR
#' @param AF_total_colname a string containing the exact column name in 'data'
#' with the whole sample AF
#'
#' @return returns a dataframe with two columns (AF_case, AF_control) and rows
#' equal to the number of variants
#'
#' @author Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
#'
#' @references https://github.com/wolffha/CCAFE
#'
#' @seealso \url{https://github.com/wolffha/CCAFE} for further documentation
#'
#' @examples
#' library(CCAFE)
#'
#' data("sampleDat")
#' sampleDat <- as.data.frame(sampleDat)
#'
#' nCase_sample = 16550
#' nControl_sample = 403923
#'
#' # get the estimated case and control AFs
#' af_method_results <- CaseControl_AF(data = sampleDat,
#'                                     N_case = nCase_sample,
#'                                     N_control = nControl_sample,
#'                                     OR_colname = "OR",
#'                                     AF_total_colname = "true_maf_pop")
#'
#' head(af_method_results)
#'
#' @export
CaseControl_AF <- function(data,
                           N_case = 0,
                           N_control = 0,
                           OR_colname = "OR",
                           AF_total_colname = "AF"){

  data <- as.data.frame(data)
  # do input checking

  # check valid input for case/control sample size
  if(N_case <= 0) {
    stop("'N_case' needs to be a number > 0")
  }

  if(N_control <= 0) {
    stop("'N_control' needs to be a number > 0")
  }

  # check valid input data type
  if(!is.data.frame(data)) {
    stop("'data' must be a dataframe")
  }

  # check that OR and AF_total columns exist
  if(!OR_colname %in% colnames(data)) {
    stop("'OR_colname' does not exist in 'data'")
  }

  if(!AF_total_colname %in% colnames(data)) {
    stop("'AF_total_colname' does not exist in 'data'")
  }

  OR <- data[,c(OR_colname)]
  AF_total <- data[,AF_total_colname]

  # check for valid input for OR and AF_total
  if(typeof(OR) != "double") {
    stop("'OR' values must all be numbers (hint: check for NAs)")
  }

  if(typeof(AF_total) != "double") {
    stop("'AF_total' values must all be numbers (hint: check for NAs)")
  }

  if(any(AF_total < 0)) {
    stop("'AF_total' cannot contain negative AFs")
  }

  if(any(AF_total > 1)) {
    stop("'AF_total' cannot contain values > 1")
  }

    #calculate total sample size
  N_total <- N_control+N_case

  #set a, b, c of quadratic equation derived in manuscript
  a <- (N_control/N_case)*(OR-1)
  b <- (OR-((N_total/N_case)*AF_total*OR))+((N_control/N_case)+
                                              (N_total*AF_total/N_case))
  c <- -(N_total/N_case)*AF_total

  # determine AF_control by applying quadratic formula, selecting root [0,1]
  # as AF_control
  AF_control <- vapply(seq_along(a), function(i) {
    AF_control_opts <- quad_roots(a[i], b[i], c[i])
    if (AF_control_opts[1] > 1 | AF_control_opts[1] < 0) {
      return(AF_control_opts[2])
    } else {
      return(AF_control_opts[1])
    }
  }, c(0.0))

  #calculate AF_case with known relationship shown in manuscript
  AF_case <- (N_total/N_case)*AF_total - (N_control/N_case)*AF_control

  data$AF_case <- AF_case
  data$AF_control <- AF_control

  #Output shows case AF first, then control AF
  return(data)
}

# solve for real roots of a quadratic using the quadratic formula
quad_roots<-function(a,b,c){
  c(((-b-sqrt(b^2-4*a*c))/(2*a)),((-b+sqrt(b^2-4*a*c))/(2*a)))
}

data("sampleDat")

sampleDat <- as.data.frame(sampleDat)

expected_mafcase_SE <- c(1.764905e-01, 3.263835e-01, 1.392353e-01, 2.513875e-06, 1.946826e-05, 1.250050e-02)
expected_mafcontrol_SE <- c(1.764405e-01, 3.266896e-01, 1.387297e-01, 2.794421e-06, 6.821993e-05, 1.269041e-02)
expected_mafpop_SE <- c(1.764425e-01, 3.266776e-01, 1.387496e-01, 2.783379e-06, 6.630105e-05, 1.268294e-02)

expected_mafcase_adj <- c(0.20818449, 0.45723980, 0.16577908, 0.00000000, 0.00000000, 0.00963508)
expected_mafcontrol_adj <- c(0.208134479, 0.457545911, 0.165273492, 0.000000000, 0.000000000, 0.009824987)
expected_mafpop_adj <- c(0.208136448, 0.457533862, 0.165293392, 0.000000000, 0.000000000, 0.009817512)

nCase_sample = 16550
nControl_sample = 403923

res_SE <- CaseControl_SE(data = sampleDat,
                         N_case = nCase_sample,
                         N_control = nControl_sample,
                         OR_colname = "OR",
                         SE_colname = "SE",
                         chromosome_colname = "CHR",
                         position_colname = "POS",
                         do_correction = F,
                         sex_chromosomes = F)

corr_dat <- data.frame(CHR = sampleDat$CHR,
                       POS = sampleDat$POS,
                       proxy_MAF = sampleDat$gnomad_maf)

res_SE_adj <- CaseControl_SE(data = sampleDat,
                             N_case = nCase_sample,
                             N_control = nControl_sample,
                             OR_colname = "OR",
                             SE_colname = "SE",
                             chromosome_colname = "CHR",
                             position_colname = "POS",
                             do_correction = T,
                             sex_chromosomes = F,
                             correction_data = corr_dat)

test_that("Number variants retained CaseControl_SE", {
  expect_equal(nrow(res_SE), 500)
})

test_that("Number variants retained CaseControl_SE with correction", {
  expect_equal(nrow(res_SE_adj), 500)
})

test_that("Correct number of columns CaseControl_SE", {
  expect_equal(ncol(res_SE), (ncol(sampleDat) + 3))
})

test_that("Correct number of columns CaseControl_SE with correction", {
  expect_equal(ncol(res_SE_adj), (ncol(sampleDat) + 6))
})

test_that("Output correct cases CaseControl_SE", {
  expect_equal(round(head(res_SE$MAF_case), 3), round(expected_mafcase_SE, 3))
})

test_that("Output correct controls CaseControl_SE", {
  expect_equal(round(head(res_SE$MAF_control), 3), round(expected_mafcontrol_SE, 3))
})

test_that("Output correct pop CaseControl_SE", {
  expect_equal(round(head(res_SE$MAF_total), 3), round(expected_mafpop_SE, 3))
})

testDat <- data.frame(OR = c(1, NA, 1, 1),
                      SE = c(.1, .2, .3, .4),
                      CHR = c(1, 1, 1, 1),
                      POS = c(1, 2, 3, 4))
test_that("Get error from NA in OR" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

testDat <- data.frame(OR = c(1, 1, 1, 1),
                      SE = c(.1, .2, NA, .4),
                      CHR = c(1, 1, 1, 1),
                      POS = c(1, 2, 3, 4))
test_that("Get error from NA in SE" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

testDat <- data.frame(OR = c(1, 1, 1, 1),
                      SE = c(.1, .2, .3, .4),
                      CHR = c(1, 1, 1, 1),
                      POS = c(1, 2, 3, 4))
test_that("Get error negative case sample size" , {
  expect_error(CaseControl_AF(data = testDat, N_case = -10, N_control = 10, OR_colname = "OR", SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

test_that("Get error negative control sample size" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = -10, OR_colname = "OR",
                              SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

testDat <- data.frame(OR = c(1, 1, 1, 1),
                      SE = c(-.1, .2, .3, .4),
                      CHR = c(1, 1, 1, 1),
                      POS = c(1, 2, 3, 4))
test_that("Get error negative SE" , {
  expect_error(CaseControl_SE(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR",
                              SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

testDat <- data.frame(OR = c(1, 1, 1, 1),
                      SE = c(.1, .2, .3, .4),
                      CHR = c(1, 1, 1, 1),
                      POS = c(1, 2, 3, 4))
test_that("Get error if odds ratio column name is wrong", {
  expect_error(CaseControl_SE(data = testDat, N_case = 10, N_control = 10, OR_colname = "Odds.Ratio",
                              SE_colname = "SE",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

test_that("Get error if standard error columnn name is wrong", {
  expect_error(CaseControl_SE(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR",
                              SE_colname = "err",
                              chromosome_colname = "CHR",
                              position_colname = "POS",
                              do_correction = F,
                              sex_chromosomes = F))
})

data("sampleDat")

sampleDat <- as.data.frame(sampleDat)

expected_afcase_AF <- c(2.041813e-01, 4.468049e-01, 1.647775e-01, 0.000000e+00, 1.077692e-05, 1.381395e-02)
expected_afcontrol_AF <- c(2.041254e-01, 4.471490e-01, 1.641968e-01, 0.000000e+00, 3.776488e-05, 1.402353e-02)

nCase_sample = 16550
nControl_sample = 403923


res_AF <- CaseControl_AF(data = sampleDat,
                         N_case = nCase_sample,
                         N_control = nControl_sample,
                         OR_colname = "OR",
                         AF_total_colname = "true_maf_pop")

test_that("Number variants retained CaseControl_AF", {
  expect_equal(nrow(res_AF), 500)
})

test_that("Correct number of columns CaseControl_AF", {
  expect_equal(ncol(res_AF), (ncol(sampleDat) + 2))
})

test_that("Output correct cases CaseControl_AF", {
  expect_equal(round(head(res_AF$AF_case),3), round(expected_afcase_AF, 3))
})

test_that("Output correct controls CaseControl_AF", {
  expect_equal(round(head(res_AF$AF_control), 3), round(expected_afcontrol_AF, 3))
})

testDat = data.frame(OR = c(1, 1, 1, 1),
                     AF = c(.1, .2, .3, .4))
test_that("Get no error from proper input" , {
  expect_no_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(OR = c(1, NA, 1, 1),
                     AF = c(.1, .2, .3, .4))
test_that("Get error from NA in OR" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(OR = c(1, 1, 1, 1),
                     AF = c(.1, NA, .3, .4))
test_that("Get error from NA in AF_population" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(OR = c(1, 1, 1, 1),
                     AF = c(.1, .2, .3, .4))
test_that("Get error negative case sample size" , {
  expect_error(CaseControl_AF(data = testDat, N_case = -10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

test_that("Get error negative control sample size" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = -10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(OR = c(1, 1, 1, 1),
                     AF = c(-.1, .2, .3, .4))
test_that("Get error negative AF" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(OR = c(1, 1, 1, 1),
                     AF = c(1.1, .2, .3, .4))
test_that("Get error AF > 1" , {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR", AF_total_colname = "AF"))
})

testDat = data.frame(odds_ratio = c(1, 1, 1, 1),
                     allele_frequency = c(.1, .2, .3, .4))
test_that("No errors if all columns present", {
  expect_no_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "odds_ratio",
                                 AF_total_colname = "allele_frequency"))
})

test_that("Get error if odds ratio column name is wrong", {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "OR",
                              AF_total_colname = "allele_frequency"))
})

test_that("Get error if allele frequency columnn name is wrong", {
  expect_error(CaseControl_AF(data = testDat, N_case = 10, N_control = 10, OR_colname = "odds_ratio",
                              AF_total_colname = "AF"))
})

#' @title CCAFE_convertVCF
#' @description
#' Formats information from a VCF object for use in CCAFE methods as follows:
#' From the rowRanges object: seqnames (chromosome), ranges (position),
#; REF, ALT, RSID
#' From the geno object: ES (effect size of ALT), SE, AF (allele frequency
#' of ALT)
#'
#' @param vcf a Variant Call Format (VCF) file read in using VariantAnnotation
#' BioConductor package
#' @return a dataframe object with columns Position, RSID, Chromosome, REF,
#' ALT, beta, SE, AF, OR
#' @export
#' @author Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
#' @examples
#' library(VariantAnnotation)
#' library(CCAFE)
#'
#' # load the data
#' data("vcf_sample")
#'
#' # run the method
#' df_sample <- CCAFE_convertVCF(vcf_sample)
#' print(head(df_sample))
#'
#' # can then use in CCAFE methods
#' # since we have total AF, will use CaseControl_AF
#' df_sample <- CaseControl_AF(data = df_sample,
#'                           N_case = 48286,
#'                           N_control = 250671,
#'                           OR_colname = "OR",
#'                           AF_total_colname = "AF")
#' head(df_sample)

CCAFE_convertVCF <- function(vcf) {
  # make sure object is VariantAnnotation VCF
  if(!inherits(vcf, "CollapsedVCF") && !inherits(vcf, "ExpandedVCF")) {
    stop("The input object is not a valid VCF (CollapsedVCF or ExandedVCF)
    from the VariantAnnotation package")
  } else {
    message("Valid VCF object, converting to dataframe for CCAFE...")
  }

  # first we will get the info from GRanges object (position, RSID)
  meta <- as.data.frame(IRanges::ranges(vcf))
  meta <- meta[,c(1, 4)]
  colnames(meta) <- c("Position", "RSID")
  meta$Chromosome <- as.vector(GenomicRanges::seqnames(
    SummarizedExperiment::rowRanges(vcf)))

  # now we can also get the meta data (REF, ALT) from the GRanges object
  meta <- cbind(meta, S4Vectors::mcols(vcf)[,c(2,3)])
  rownames(meta) <- seq(1, nrow(meta))

  # now we will get the info from the geno object
  geno_dat <- data.frame(
    beta = unlist(VariantAnnotation::geno(vcf)$ES),
    SE = unlist(VariantAnnotation::geno(vcf)$SE),
    AF = unlist(VariantAnnotation::geno(vcf)$AF)
  )

  df_data <- cbind(meta, geno_dat)

  # use OR (not beta) in CCAFE so we will create that column
  df_data$OR <- exp(df_data$beta)

  df_data <- as.data.frame(df_data)

  return(df_data)
}

#' @title CaseControl_SE
#' @description This is a function to derive the case, control, and total MAFs
#' from GWAS summary statistics when the user has access to the sample sizes,
#' and the OR (or beta), and SE for the log(OR) for each variant.
#' If user has total AF instead of SE use [CCAFE::CaseControl_AF()]
#' This code uses the GroupFreq function adapted from C from
#' <https://github.com/Paschou-Lab/ReAct/blob/main/GrpPRS_src/CountConstruct.c>
#'
#' @param data dataframe where each row is a variant and columns contain the
#' OR, SE, chromosome and positions
#' @param N_case an integer of the number of Case individuals
#' @param N_control an integer of the number of Control individuals
#' @param OR_colname a string containing the exact column name in 'data' with
#' the OR
#' @param SE_colname a string containing the exact column name in 'data' with
#' the SE
#' @param chromosome_colname a string containing the exact column name in 'data'
#' with the chromosomes, default "chr"
#' @param position_colname a string containing the exact column name in 'data'
#' with the position, default "pos"
#' @param sex_chromosomes boolean, TRUE if variants from sex chromosomes are
#' included in the dataset. Sex chromosomes can be numeric (23, 24) or
#' character (X, Y). If numeric, assumes X=23 and Y=24.
#' @param N_XX_case the number of XX chromosome case individuals
#' (REQUIRED if sex_chromosomes == TRUE)
#' @param N_XX_control the number of XX chromosome control individuals
#' (REQUIRED if sex_chromosomes == TRUE)
#' @param N_XY_case the number of XY chromosome case individuals
#' (REQUIRED if sex_chromosomes == TRUE)
#' @param N_XY_control the number of XY chromosome control individuals
#' (REQUIRED if sex_chromosomes == TRUE)
#' @param do_correction boolean, TRUE if data is provided to perform correction
#' @param correction_data a dataframe with the following exact columns: CHR,
#' POS, proxy_MAF with data that is harmonized between the proxy true datasets
#' and the observed dataset
#' @param remove_sex_chromosomes boolean, TRUE if should keep autosomes only.
#' This is needed when the number of biological sex males/females per case and
#' control group is not known.
#' @param verbose boolean, determine whether warnings should be displayed
#' (default FALSE)
#'
#' @return returns data as a dataframe with three additional columns: MAF_case,
#' MAF_control, MAF_total for the estimated MAFs for each variant. If
#' do_correction = TRUE, then will output 3 additional columns
#' (MAF_case_adj, MAF_control_adj, MAF_total_adj) with the adjusted estimates.
#'
#' @author Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
#'
#' @references https://github.com/wolffha/CCAFE
#'
#' @seealso \url{https://github.com/wolffha/CCAFE} for further documentation
#'
#' @examples
#' library(CCAFE)
#'
#' data("sampleDat")
#' sampleDat <- as.data.frame(sampleDat)
#'
#' nCase_sample = 16550
#' nControl_sample = 403923
#'
#' # get the estimated case and control MAFs
#' se_method_results <- CaseControl_SE(data = sampleDat,
#'                                     N_case = nCase_sample,
#'                                     N_control = nControl_sample,
#'                                     OR_colname = "OR",
#'                                     SE_colname = "SE",
#'                                     chromosome_colname = "CHR",
#'                                     position_colname = "POS")
#'
#' head(se_method_results)
#'
#' @importFrom dplyr left_join filter
#' @export
CaseControl_SE <- function(data, N_case = 0, N_control = 0, OR_colname = "OR",
                           SE_colname = "SE",
                           chromosome_colname = "chr", sex_chromosomes = FALSE,
                           position_colname = "pos",
                           N_XX_case = NA,
                           N_XX_control = NA,
                           N_XY_case = NA,
                           N_XY_control = NA,
                           do_correction = FALSE,
                           correction_data = NA,
                           remove_sex_chromosomes = TRUE,
                           verbose = FALSE) {
  # do input checking

  data <- as.data.frame(data)

  # check valid input for case/control sample size
  if(N_case <= 0) {
    stop("'N_case' needs to be a number > 0")
  }

  if(N_control <= 0) {
    stop("'N_control' needs to be a number > 0")
  }

  # check valid input data type
  if(!is.data.frame(data)) {
    stop("'data' must be a dataframe")
  }

  # check that OR and SE columns exist
  if(!OR_colname %in% colnames(data)) {
    stop("'OR_colname' does not exist in 'data'")
  }

  if(!SE_colname %in% colnames(data)) {
    stop("'SE_colname' does not exist in 'data'")
  }

  if(!chromosome_colname %in% colnames(data)) {
    stop("'chromosome_colname does not exist in 'data'")
  }

  if(!position_colname %in% colnames(data)) {
    stop("'position_colname does not exist in 'data'")
  }

  OR <- data[,OR_colname]
  SE <- data[,SE_colname]

  # check for valid input for OR and SE
  if(typeof(OR) != "double") {
    stop("'OR' values must all be numbers (hint: check for NAs)")
  }

  if(typeof(SE) != "double") {
    stop("'SE' values must all be numbers (hint: check for NAs)")
  }

  if(any(SE < 0)) {
    stop("'SE' cannot contain negative values")
  }

  sexchr_type <- NA

  # check sex chromosome info
  if(typeof(data[,c(chromosome_colname)]) == "character") {
    # check if can be coerced to all numeric
    if(any(suppressWarnings(is.na(as.numeric(data[,c(chromosome_colname)]))))) {
      sexchr_type <- "chr"
      # there were characters in chromosome column so make sure sex chromosomes
      # is true
      if(!sex_chromosomes & !remove_sex_chromosomes) {
        stop("Non-numeric chromosomes (sex chromosomes) present but
             'sex_chromosomes' = FALSE")
      }
    } else { # if there aren't any non-numeric
      # make sure there are none > 22 which could be numeric representations
      # of sex chromosomes
      if(any(as.numeric(data[,c(chromosome_colname)]) > 22)) {
        if(!sex_chromosomes & !remove_sex_chromosomes) {
          sexchr_type <- "num"
          stop("Chromosomes > 22 (sex chromosomes) present but
               'sex_chromosomes' = FALSE")
        }
      }
    }
  } else {
    # if chromosome column is all numeric check for those > 22 which are
    # likely sex chromosomes
    if(any(data[,c(chromosome_colname)]) > 22) {
      sexchr_type <- "num"
      if(!sex_chromosomes) {
        stop("Chromosomes > 22 (sex chromosomes) present but
             'sex_chromosomes' = FALSE")
      }
    }
  }

  # make sure N_XX and N_XY are included if sex_chromosomes is true
  if(sex_chromosomes & !remove_sex_chromosomes) {
    if(is.na(N_XX_case) | is.na(N_XX_control) | is.na(N_XY_case) |
       is.na(N_XY_control)) {
      stop("'sex_chromosomes' = TRUE but N_XX_case, N_XX_control, N_XY_case and
      N_XY_control are not included")
    }
  }
  # input checking for correction
  if(do_correction) {
    if(!is.data.frame(correction_data)) {
      stop("'correction_data' must be a dataframe")
    }

    if(!"CHR" %in% colnames(correction_data) |
       ! "POS" %in% colnames(correction_data) |
       ! "proxy_MAF" %in% colnames(correction_data)) {
      stop("column names 'CHR', 'POS', and 'proxy_MAF' must be in
           'correction_data'")
    }
  }


  if(!sex_chromosomes) {
    # this function uses w, x, y, z as the derivation in the ReACt paper does
    data <- return_maf(SE, N_case, N_control, OR, data)

  } else {
    if(remove_sex_chromosomes) {
      keep_chrs <- seq(1, 22, 1)
      data$temp_chr <- data[,c(chromosome_colname)]
      data <- data[data$temp_chr %in% keep_chrs, ]

      SE <- data[,c(SE_colname)]
      OR <- data[,c(OR_colname)]

      data <- return_maf(SE, N_case, N_control, OR, data)

    } else {
      data$temp_chr <- data[,c(chromosome_colname)]

      N_XX_total <- N_XX_case + N_XX_control
      N_XY_total <- N_XY_case + N_XY_control
      N_total <- N_case + N_control

      if(sexchr_type == "num") {
        data_nosex <- data[data$temp_chr <= 22, ]
        data_x <- data[data$temp_chr == 23, ]
        data_y <- data[data$temp_chr == 24, ]
      } else {
        data_nosex <- data[!data$temp_chr %in% c("X", "x", "Y", "y"), ]
        data_x <- data[data$temp_chr %in% c("X", "x"), ]
        data_y <- data[data$temp_chr %in% c("Y", "y"), ]

      }
      # do autosome data
      SE <- data_nosex[,c(SE_colname)]
      OR <- data_nosex[,c(OR_colname)]

      data_nosex <- return_maf(SE, N_case, N_control, OR, data_nosex)

      # do sex chromosomes
      # start with x
      if(nrow(data_x) > 0) {
        SE <- data_x[,c(SE_colname)]
        OR <- data_x[,c(OR_colname)]

        N_X_control <- (2*N_XX_control + N_XY_control)
        N_X_case <- (2*N_XX_case + N_XY_case)
        N_X_total <- (2*N_XX_total + N_XY_total)

        data_x <- return_maf(SE, N_X_case, N_X_control, OR, data_x)

      }
      # do y chromosome
      if(nrow(data_y > 0)) {
        SE <- data_y[,c(SE_colname)]
        OR <- data_y[,c(OR_colname)]

        N_Y_control <- (N_XY_control)
        N_Y_case <- (N_XY_case)
        N_Y_total <- (N_XY_total)

        data_y <- return_maf(SE, N_Y_case, N_Y_control, OR, data_y)

      }
      data <- rbind(data_nosex, data_x, data_y)
    }
  }


  # do the correction if do_correction
  if(!do_correction) {
    return(data)
  } else {
    # Merge and apply bias correction
    if(verbose) {
      merged <- tryCatch(
        left_join(correction_data, data, by = c("CHR" = chromosome_colname,
                                                "POS" = position_colname)),
        warning = function(w) {
          message("In left_join: ", conditionMessage(w))
          return(left_join(correction_data, data,
                           by = c("CHR" = chromosome_colname,
                                  "POS" = position_colname)))
        }
      )
    } else {
      merged <- suppressWarnings(left_join(correction_data, data,
                                           by = c("CHR" = chromosome_colname,
                                                  "POS" = position_colname)))
    }


    mod <- get_model(estimated = merged$MAF_total, true = merged$proxy_MAF)
    bias <- get_bias(mod, estimated = data$MAF_total)

    data <- within(data, {
      MAF_case_adj <- MAF_case + bias
      MAF_control_adj <- MAF_control + bias
      MAF_total_adj <- MAF_total + bias
    })

    return(data)
  }
}

# solve for real roots of a quadratic using the quadratic formula
quad_roots<-function(a,b,c){
  c(((-b-sqrt(b^2-4*a*c))/(2*a)),((-b+sqrt(b^2-4*a*c))/(2*a)))
}

# Function contains code to calculate the MAF using code adapted from ReACt.
# For explanations of w, x, y, z, see original ReACt paper (Supplement 5.2)
solve_maf <- function(w, x, y, z, N_case, N_control) {
  AC_control <- rep(0, length(w))
  AC_case <- rep(0, length(w))

  for(i in seq_len(length(w))) {
    # need to make sure the discriminate will be positive or it won't solve
    # inflate w(se) by 1.001, for at max 49 times (usually can be done within
    # 5 iterations)
    #maximum inflation is 0.050195, ~5%), if disc still < 0 then give up
    for(j in seq(from = 0, to = 99, by = 1)) {
      w[i] <- w[i] * (1.001^j)
      disc <- (((2*z[i]*y*(1-z[i])) - (w[i]*x*y*z[i]))^2) - 4*(w[i]*x*z[i] +
                                                                 (1-z[i])^2)*(y*z[i]*(x + y*z[i]))
      if (!is.na(disc) & disc >= 0) {
        break
      }
    }
    if (is.na(disc) | disc < 0) { # if discriminate is <0 cannot solve; return 0s
      AC_control[i] <- 0
      AC_case[i] <- 0
    } else { # now actually solve the quadratic for allele counts (AC)
      # solve for the a, b, and c of the quadratic equation
      # this quadratic is solving for the allele count (AC) of the controls
      # overall their derivation relies on AC (not AF) and then calculates AF
      a <- (w[i]*x*z[i]) + (1-z[i])^2
      b <- 2*y*z[i]*(1-z[i]) - w[i]*x*y*z[i]
      c <- y*z[i]*(x + y*z[i])

      #find roots of quadratic equation
      AF_control_opts <- suppressWarnings(quad_roots(a, b, c))
      # in order to select which root, we need to use each option (d1 and d2)
      # to calculate a, b, c of the 2x2 table of allele counts
      d1 <- AF_control_opts[1]
      c1 <- y - d1
      b1 <- (x*d1)/(y*z[i] - z[i]*d1 + d1)
      a1 <- x - b1

      d2 <- AF_control_opts[2]
      c2 <- y - d2
      b2 <- (x*d2)/(y*z[i] - z[i]*d2 + d2)
      a2 <- x - b2

      vec1 <- c(a1, b1, c1, d1) # vector of a,b,c,d using root 1
      vec2 <- c(a2, b2, c2, d2) # vector of a,b,c,d using root 2

      # if both roots allow for all values to be positive, choose the larger
      if(!any(vec1 < 0) & !(any(vec2 < 0))) {
        # if d1 is the larger root, then the AC_control = c1 and AC_case = a1
        if(d1 > d2) {
          AC_control[i] <- c1
          AC_case[i] <- a1
        } else {
          AC_control[i] <- c2
          AC_case[i] <- a2
        }
        # if d1 allows all values of 2x2 to be positive but NOT d2 use c1 and a1
      } else if(!any(vec1 < 0)) {
        AC_control[i] <- c1
        AC_case[i] <- a1
        # if d2 allows all values to be positive but NOT d1, use c2 and a2
      } else {
        AC_control[i] <- c2
        AC_case[i] <- a2
      }
    }
  }

  # calculate and return the MAF
  MAF_case <- AC_case/(x)
  MAF_control <- AC_control/(y)
  MAF_pop <- (AC_case + AC_control)/(x + y)

  return(data.frame(MAF_case, MAF_control, MAF_pop))
}

# function to set w, x, y, z and call solve MAF and add it to 'data' dataframe
return_maf <- function(SE, N_case, N_control, OR, data) {
  w <- SE^2
  x <- 2*N_case
  y <- 2*N_control
  z <- OR

  res <- solve_maf(w, x, y, z, N_case, N_control)
  data$MAF_case <- res$MAF_case
  data$MAF_control <- res$MAF_control
  data$MAF_total <- res$MAF_pop
  return(data)
}

# function to get the correction/bias model
# cannot be vectorized as each row calcuation depends on previous rows' values
get_model <- function(bins = "large", estimated, true) {
    if(bins == "large") {
      binDat <- data.frame(bins = c("[0.0, 0.1)", "[0.1, 0.2)", "[0.2, 0.3)",
                                    "[0.3, 0.4)", "[0.4, 0.5]"),
                           min = c(0, 0.1, 0.2, 0.3, 0.4),
                           max = c(0.1, 0.2, 0.3, 0.4, 0.5),
                           x = rep(0, 5),
                           x2 = rep(0,5),
                           intercept = rep(0, 5),
                           ymin = rep(0, 5),
                           ymax = rep(0, 5))
    } else {
      binDat <- data.frame(bins = c("[0.0, 0.05)", "[0.05, 0.1)", "[0.1, 0.15)",
                                    "[0.15, 0.2)","[0.2, 0.25)", "[0.25, 0.3)",
                                    "[0.3, 0.35)", "[0.35, 0.4)",
                                    "[0.4, 0.45)", "[0.45, 0.5]"),
                           min = c(0, 0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35,
                                   0.4, 0.45),
                           max = c(0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4,
                                   0.45, 0.5),
                           x = rep(0, 10),
                           x2 = rep(0, 10),
                           intercept = rep(0, 10),
                           ymin = rep(0, 10),
                           ymax = rep(0, 10))
    }

    # store the models and adjusted MAFs
    dat <- data.frame(estimated = estimated, true = true)
    adjusted <- rep(0, length(estimated))

    for(i in seq_len(nrow(binDat))) {
      # filter data for model fitting to those within MAF bin
      subdat <- dplyr::filter(dat, true >= binDat[i,]$min &
                                true < binDat[i,]$max)
      mod <- stats::lm(data = subdat,
                       formula = estimated ~ stats::poly(true, 2, raw = TRUE))

      # get studentized residuals use to filter outliers and refit model
      stud_res <- mod$residuals * stats::sd(mod$residuals)
      tokeep <- which(abs(stud_res) < 3)

      if(length(tokeep) > 0) {
        mod2 <- stats::lm(data = subdat[tokeep,],
                          formula = estimated ~ stats::poly(true,2, raw = TRUE))

        # save refit model coefficients
        binDat[i,]$intercept <- mod2$coefficients[1]
        binDat[i,]$x <- mod2$coefficients[2]
        binDat[i,]$x2 <- mod2$coefficients[3]
      } else {
        # save refit model coefficients
        binDat[i,]$intercept <- mod$coefficients[1]
        binDat[i,]$x <- mod$coefficients[2]
        binDat[i,]$x2 <- mod$coefficients[3]
      }

      # calculate max y'
      binDat[i,]$ymax <- binDat[i,]$max^2*binDat[i,]$x2 +
        binDat[i,]$max*binDat[i,]$x + binDat[i,]$intercept

      # assign y' min
      if(i == 1) {
        binDat[i,]$ymin <- 0
      } else {
        binDat[i,]$ymin <- binDat[i-1, ]$ymax
      }
    }
    return(binDat)
  }

# function to calculate estimated bias
get_bias <- function(binDat, estimated){
  # first create dataframe with estimated, and the appropriate intercept, x, x2
  # values given the value of estimated
  dat <- data.frame(estimated = estimated)

  breaks <- c(0, binDat$ymax, 1)
  dat$bin <- cut(dat$estimated, breaks = breaks, include.lowest = TRUE,
                 right = FALSE, labels = FALSE)
  dat[dat$bin == 6, ]$bin <- 5

  dat$intercept <- 0
  dat$x2 <- 0
  dat$x <- 0

  for(i in seq(1, 5, 1)) {
    dat[dat$bin == i, ]$intercept <- binDat[i, ]$intercept
    dat[dat$bin == i, ]$x <- binDat[i,]$x
    dat[dat$bin == i, ]$x2 <- binDat[i, ]$x2
  }
  # for each variant calculate the value of x for the regression (gnomAD value
  # the corresponds to that estimated value)
  get_x <- function(dat) {
    # we have only AF' which, in terms of fitting the regression = y'
    # need to calculate x given y'
    # y' = beta_0 + beta_1*x + beta_2*x^2 (solve the quadratic)

    # data columns will be in the following order:
    # estimated, bin, intercept, x2, x
    x_opts <- suppressWarnings(quad_roots(dat[4], dat[5], (dat[3] - dat[1])))
    if(all(is.na(x_opts))) {
      x <- 0
    } else {
      if(!any(x_opts >= 0 & x_opts <= 0.5)) {
        x <- 0
      } else {
        x <- x_opts[which(x_opts >= 0 & x_opts <= .5)]
        if(length(x) > 1) { # if both are between 0 and .5 then select the one
                            # closest to the estimated value
          x <- x[which.min(abs(x - dat[1]))]
        }
      }
    }
    return(as.numeric(x))
  }

  dat$gnomad <- apply(dat, 1, function(x) unlist(get_x(x)))

  # calculate the bias
  bias <- dat$gnomad - dat$estimated

  # anything from 0 to 0.05 don't correct
  #bias[which(estimated < 0.05)] <- 0
  # if 0 output as gnomAD estimate don't correct
  bias[which(dat$gnomad == 0)] <- 0

  return(bias)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("CCAFE")

## ----eval=FALSE---------------------------------------------------------------
# if(!require(devtools, quietly = TRUE)) {
#   install.packages("devtools")
# }
# 
# devtools::install_github("https://github.com/wolffha/CCAFE")

## ----echo=TRUE,message=FALSE--------------------------------------------------
library(CCAFE)
library(tidyverse)

## ----CaseControl_AF example---------------------------------------------------
# load the data
data("sampleDat")
sampleDat <- as.data.frame(sampleDat)

results_af <- CaseControl_AF(data = sampleDat,
                             N_case = 16550,
                             N_control = 403923,
                             OR_colname = "OR",
                             AF_total_colname = "true_maf_pop")

head(results_af)

## ----eval=TRUE----------------------------------------------------------------
# plot the results
# first need minor allele frequency to compare to true_maf_case
results_af$MAF_case <- sapply(results_af$AF_case, function(x)
  ifelse(x > 0.5, 1-x, x))
results_af <- as.data.frame(results_af)

ggplot(results_af, aes(x = true_maf_case, y = MAF_case)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  ggtitle("Example CaseControl_AF()")

## ----CaseControl_SE example - no correction-----------------------------------
# load the data
data("sampleDat")

# always ensure data is a dataframe before inputting to CCAFE methods
sampleDat <- as.data.frame(sampleDat)

# First run without correction
results_se_noCorr <- CaseControl_SE(data = sampleDat,
                                    N_case = 16550,
                                    N_control = 403923,
                                    OR_colname = "OR",
                                    SE_colname = "SE",
                                    chromosome_colname = "CHR",
                                    position_colname = "POS",
                                    do_correction = FALSE,
                                    sex_chromosomes = FALSE)
head(results_se_noCorr)

## ----eval=TRUE----------------------------------------------------------------
# plot the results
ggplot(results_se_noCorr, aes(x = true_maf_case, y = MAF_case)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  coord_cartesian(ylim = c(0,.5)) +
  ggtitle("Example CaseControl_SE() No Correction")

## ----CaseControl_SE example - correction--------------------------------------
# load the data
data("sampleDat")
# always ensure data is a dataframe before inputting to CCAFE methods
sampleDat <- as.data.frame(sampleDat)

corr_data <- data.frame(CHR = sampleDat$CHR,
                        POS = sampleDat$POS,
                        proxy_MAF = sampleDat$gnomad_maf)

# now run with correction
results_se_corr <- CaseControl_SE(data = sampleDat,
                                  N_case = 16550,
                                  N_control = 403923,
                                  OR_colname = "OR",
                                  SE_colname = "SE",
                                  chromosome_colname = "CHR",
                                  position_colname = "POS",
                                  do_correction = TRUE,
                                  correction_data = corr_data,
                                  sex_chromosomes = FALSE)

head(results_se_corr)

## ----eval=TRUE----------------------------------------------------------------
# plot the results
ggplot(results_se_corr, aes(x = true_maf_case, y = MAF_case_adj)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  ggtitle("Example CaseControl_SE() With Correction")

## ----eval=FALSE---------------------------------------------------------------
# BiocManager::install("VariantAnnotation")

## -----------------------------------------------------------------------------
suppressWarnings(suppressPackageStartupMessages({
  library(VariantAnnotation)
  library(tidyverse)
}))

## -----------------------------------------------------------------------------
data("vcf_sample")

## -----------------------------------------------------------------------------
samples(header(vcf_sample))

## -----------------------------------------------------------------------------
vcf_sample

## -----------------------------------------------------------------------------
# first we will get the info from GRanges object (position, RSID)
meta <- as.data.frame(ranges(vcf_sample))
meta <- meta[,c(1, 4)]
colnames(meta) <- c("Position", "RSID")
# get the chromosome (as a)
meta$Chromosome <- as.vector(seqnames(rowRanges(vcf_sample)))

# now we can also get the meta data (REF, ALT) from the GRanges object
meta <- cbind(meta, mcols(vcf_sample)[,c(2,3)])
rownames(meta) <- seq(1, nrow(meta))

# now we will get the info from the geno object
geno_dat <- data.frame(
  beta = unlist(geno(vcf_sample)$ES),
  SE = unlist(geno(vcf_sample)$SE),
  AF = unlist(geno(vcf_sample)$AF)
)

df_data <- cbind(meta, geno_dat)
head(df_data)

## -----------------------------------------------------------------------------
df_data$OR <- exp(df_data$beta)

## -----------------------------------------------------------------------------
df_data <- CaseControl_AF(data = df_data,
                              N_case = 48286,
                              N_control = 250671,
                              OR_colname = "OR",
                              AF_total_colname = "AF")
head(df_data)

## -----------------------------------------------------------------------------
plotdata_AF <- df_data %>% dplyr::select(AF, AF_case, AF_control)
plotdata_AF$AF_total <- (plotdata_AF$AF_case*48286 + plotdata_AF$AF_control*250671)/(298857)
colnames(plotdata_AF)[1] <- "AF(known)"
plotdata_AF_long <- pivot_longer(plotdata_AF, cols = colnames(plotdata_AF))

## -----------------------------------------------------------------------------
ggplot(plotdata_AF_long, aes(x = value, fill = name)) +
  geom_boxplot(alpha = 0.5, outliers = FALSE) +
  facet_wrap(~name, nrow = 4) +
  theme_bw()

## ----CCAFE_convertVCF Demo----------------------------------------------------
df_data_2 <- CCAFE_convertVCF(vcf_sample)

## -----------------------------------------------------------------------------
df_data_2 <- CaseControl_AF(data = df_data_2,
                              N_case = 48286,
                              N_control = 250671,
                              OR_colname = "OR",
                              AF_total_colname = "AF")
head(df_data_2)

## -----------------------------------------------------------------------------
sessionInfo()

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @title CCAFE: Case and Control Allele Frequency Estimation
#' @name CCAFE
#' @importFrom dplyr filter mutate select left_join
#' @section Main functions:
#' - [CaseControl_AF()]
#'
#' - [CaseControl_SE()]
#' @section Utility functions:
#' - [CCAFE_convertVCF()]
#'
## usethis namespace: end
NULL

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)

## ----echo=TRUE, message=FALSE-------------------------------------------------
library(tidyverse)
library(CCAFE)
library(DescTools)
library(cowplot)

## ----Simulations--------------------------------------------------------------
# need to create simulated AFs from autosomes and sex chromosomes

# simulate 100 variants from each chromosome

# will simulate from a sample of 5000 XX and 5000 XY case individuals
# then add 5000 of each in control

set.seed(2020)

N_XX <- 5000
N_XY <- 5000
N <- N_XX + N_XY

for(chr in 1:24) {
  refs <- runif(n = 100, min = 0, max = 1)
  if(chr < 23) {
    genos <- t(sapply(refs, function(x) {
      rmultinom(1, N, c(x^2, 2*x*(1-x), (1-x)^2))
    }))
    afs_toadd <- apply(genos, 1, function(x) ((x[1]*2)+x[2])/(2*N))
  } else {
    if(chr == 23) { # simulate X chromosome variants
      XX_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XX*2, 1, x))
      }))
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      total_alleles <- XX_alleles + XY_alleles
      afs_toadd <- total_alleles/(2*N_XX + N_XY)
    } else { # simulate Y chromosome variants
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      afs_toadd <- XY_alleles/(N_XY)
    }
  }
  if(chr == 1) {
    simDat <- data.frame(chr = chr, pos = c(1:100), case_sim_af = afs_toadd)
  } else {
    simDat <- rbind(simDat, data.frame(chr = chr, pos = c(1:100), case_sim_af = afs_toadd))
  }
}

simDat$control_sim_af <- 0

# simulate control AFs

for(chr in 1:24) {
  refs <- simDat[simDat$chr == chr,]$case_sim_af
  refs <- refs - runif(n = length(refs), min = 0, max = .025)
  refs[refs < 0] <- 0
  refs[refs > 1] <- 1
  
  if(chr < 23) {
    genos <- t(sapply(refs, function(x) {
      rmultinom(1, N, c(x^2, 2*x*(1-x), (1-x)^2))
    }))
    afs_toadd <- apply(genos, 1, function(x) ((x[1]*2)+x[2])/(2*N))
  } else {
    if(chr == 23) { # simulate X chromosome variants
      XX_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XX*2, 1, x))
      }))
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      total_alleles <- XX_alleles + XY_alleles
      afs_toadd <- total_alleles/(2*N_XX + N_XY)
    } else { # simulate Y chromosome variants
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      afs_toadd <- XY_alleles/(N_XY)
    }
  }
  simDat[simDat$chr == chr, ]$control_sim_af <- afs_toadd
}

simDat$total_sim_af <- (simDat$case_sim_af*10000 + simDat$control_sim_af*10000)/(10000 + 10000)

# need to add OR and SE
OR <- rep(0, nrow(simDat))
SE <- rep(0, nrow(simDat))

for(i in 1:nrow(simDat)) {
  AF1 = simDat[i, ]$case_sim_af
  AF2 = simDat[i,]$control_sim_af
  if(simDat[i,]$chr < 23) {
    # first calculate the 2x2 tables of allele counts
    a = AF1 * 2 * 10000 
    b = (1-AF1) * 2 * 10000
    c = AF2 * 2 * 10000
    d = (1-AF2) * 2 * 10000
  } else {
    if(simDat[i,]$chr == 23) {
      a = AF1 * (2*5000 + 5000)
      b = (1-AF1) * (2*5000 + 5000)
      c = AF2 * (2*5000 + 5000)
      d = (1-AF2) * (2*5000 + 5000)
    } else {
      a = AF1 * 5000
      b = (1-AF1) * 5000
      c = AF2 * 5000
      d = (1-AF2) * 5000
    }
  }
  
  OR[i] <- (a*d)/(b*c)
  SE[i] <- sqrt(1/a + 1/b + 1/c + 1/d)
}

simDat$OR <- OR
simDat$SE <- SE

simDat[simDat$chr == 23, ]$chr <- "X"
simDat[simDat$chr == 24, ]$chr <- "Y"

simDat <- simDat %>% rowwise() %>%
  mutate(is_minor_case = ifelse(case_sim_af <= 0.5, 1, 0),
         is_minor_control = ifelse(control_sim_af <= 0.5, 1, 0),
         is_minor_total = ifelse(total_sim_af <= 0.5, 1, 0)) %>%
  mutate(case_sim_maf = ifelse(is_minor_case, case_sim_af, 1-case_sim_af),
         control_sim_maf = ifelse(is_minor_control, control_sim_af, 1-control_sim_af),
         total_sim_maf = ifelse(is_minor_total, total_sim_af, 1-total_sim_af))

head(simDat)

## ----Any mismatched alleles---------------------------------------------------
nrow(simDat[simDat$is_minor_case != simDat$is_minor_control, ])

## ----Show mismatched alleles--------------------------------------------------
diff_alleles <- simDat[simDat$is_minor_case != simDat$is_minor_control, ]
diff_alleles

## ----Calculate AFs------------------------------------------------------------
simDat <- simDat[is.finite(simDat$OR), ]
simDat <- as.data.frame(simDat)

res_AF <- CaseControl_AF(data = simDat,
                         N_case = 10000,
                         N_control = 10000,
                         OR_colname = "OR",
                         AF_total_colname = "total_sim_af")

res_MAF <- CaseControl_AF(data = simDat,
                         N_case = 10000,
                         N_control = 10000,
                         OR_colname = "OR",
                         AF_total_colname = "total_sim_maf")

## ----Plot_AF_results, message=FALSE-------------------------------------------
p_1 <- ggplot(res_AF, aes(x = case_sim_af, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True AF") +
  ylab("Estimated AF") +
  geom_text(x = .175, y = .9, label = paste0("CCC=", round(DescTools::CCC(res_AF$case_sim_af, res_AF$AF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("1. No Conversion")

res_AF$MAF_case <- sapply(res_AF$AF_case, function(x) ifelse(x > 0.5, 1-x, x))
res_AF <- as.data.frame(res_AF)
p_2 <- ggplot(res_AF, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  geom_text(x = .1, y = .45, label = paste0("CCC=", round(DescTools::CCC(res_AF$case_sim_maf, res_AF$MAF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("2. Convert AFTER")

p_3 <- ggplot(res_MAF, aes(x = case_sim_maf, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  geom_text(x = .1, y = .48, label = paste0("CCC=", round(DescTools::CCC(res_MAF$case_sim_maf, res_MAF$AF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("3. Convert BEFORE")

cowplot::plot_grid(p_1, p_2, p_3, ncol = 3)

## ----ceate unaccounted for sex chr data---------------------------------------
# to simulate a dataset in which the sex chromosomes are not properly accounted for, we will falsely
# rename the x and y chromosomes to autosomes
simDat_nosex <- simDat
simDat_nosex[simDat_nosex$chr == "X", ]$chr <- 1
simDat_nosex[simDat_nosex$chr == "Y", ]$chr <- 2

## ----Get estimated AF/MAF-----------------------------------------------------
af_res_sex <- CaseControl_AF(data = simDat,
                           N_case = 10000,
                           N_control = 10000,
                           OR_colname = "OR",
                           AF_total_colname = "total_sim_af")

se_res <- CaseControl_SE(data = simDat_nosex,
                                N_case = 10000,
                                N_control = 10000,
                                OR_colname = "OR",
                                SE_colname = "SE",
                                chromosome_colname = "chr",
                                position_colname = "pos",
                                sex_chromosomes = FALSE,
                                do_correction = FALSE)

se_res_sex <- CaseControl_SE(data = simDat,
                             N_case = 10000,
                             N_control = 10000,
                             OR_colname = "OR",
                             SE_colname = "SE",
                             chromosome_colname = "chr",
                             position_colname = "pos",
                             sex_chromosomes = TRUE,
                             remove_sex_chromosomes = FALSE,
                             do_correction = FALSE,
                             N_XX_case = 5000,
                             N_XX_control = 5000,
                             N_XY_case = 5000,
                             N_XY_control = 5000)

## ----plot_sex_chromosome_results, message=FALSE-------------------------------
p_a <- ggplot(af_res_sex, aes(x = case_sim_af, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True AF") +
  ylab("Estimated AF") +
  ggtitle("1. CaseControl_AF")

p_b <- ggplot(se_res, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  ggtitle("2. CaseControl_SE") 

p_c <- ggplot(se_res_sex, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  ggtitle("3. CaseControl_SE") 

plot_grid(p_a, p_b, p_c, ncol =3)

## -----------------------------------------------------------------------------
sessionInfo()

#' PanUKBB and gnomAD Diabetes Data
#'
#' This is a subset of 500 variants on chromosome 1 from the Pan-UKBB diabetes GWAS with the whole sample (pop), case, and control minor allele frequency (MAF) for those classified in Pan-UKBB as European (EUR). 
#' These variants (which are mapped to GRCh37) have been harmonized with gnomAD non-Finnish European (NFE) MAFs.
#'
#' @format `sampleDat`
#' A data frame with 500 rows and 11 columns:
#' \describe{
#'   \item{CHR}{chromosome number}
#'   \item{POS}{base-pair position of variant (GRCh37 coordinates)}
#'   \item{REF}{Reference allele}
#'   \item{ALT}{Alternate allele}
#'   \item{true_maf_case}{MAF in EUR cases in Pan-UKBB Diabetes}
#'   \item{true_maf_control}{MAF in EUR controls in Pan-UKBB Diabetes}
#'   \item{true_maf_pop}{MAF in whole EUR sample in Pan-UKBB Diabetes}
#'   \item{beta}{beta from EUR GWAS in Pan-UKBB Diabetes}
#'   \item{SE}{SE of beta from EUR GWAS in Pan-UKBB Diabetes}
#'   \item{OR}{OR from EUR GWAS in Pan-UKBB Diabetes}
#'   \item{gnomad_maf}{MAF in gnomAD NFE}
#' }
#' @source <https://pan.ukbb.broadinstitute.org/docs/per-phenotype-files>
#' @source <https://gnomad.broadinstitute.org/downloads>
"sampleDat"

#' A VCF from this GWAS of Type 2 Diabetes https://doi.org/10.1038/s41588-018-0084-1. containing
#' a subset of 10,000 variants
#'
#' @format `vcf_sample`
#' A CollapsedVCF
#'
#' dim: 10000 1
#' rowRanges(vcf):
#'   GRanges with 5 metadata columns: paramRangeID, REF, ALT, QUAL, FILTER
#' info(vcf):
#'   DataFrame with 1 column: AF
#' info(header(vcf)):
#'   Number Type  Description
#' AF A      Float Allele Frequency
#' geno(vcf):
#'   List of length 9: ES, SE, LP, AF, SS, EZ, SI, NC, ID
#' geno(header(vcf)):
#'   Number Type    Description
#' ES A      Float   Effect size estimate relative to the alternative allele
#' SE A      Float   Standard error of effect size estimate
#' LP A      Float   -log10 p-value for effect estimate
#' AF A      Float   Alternate allele frequency in the association study
#' SS A      Integer Sample size used to estimate genetic effect
#' EZ A      Float   Z-score provided if it was used to derive the EFFECT and SE fields
#' SI A      Float   Accuracy score of summary data imputation
#' NC A      Integer Number of cases used to estimate genetic effect
#' ID 1      String  Study variant identifier


"vcf_sample"

---
title: "CCAFE Vignette"
author: "Hayley Stoneman"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{CCAFE Vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)
```

# Introduction

*Motivation*

Growth in the field of genetic and genomic research has vastly increased the amount of available 
data which is often made publicly available through summary statistics. While there are many 
opportunities and new methods to use summary-level genetic data, it is often limited by the 
statistics that are made available. Many post-hoc analyses of disease data require case and 
control allele frequencies (AFs), which are not always published.  We present a framework to 
derive the case and control AFs from Genome Wide Association Study (GWAS) summary statistics using 
the whole sample (case and control aggregated) AF, odds ratio, and case and control sample sizes, 
and compare it to a previously published method that uses the standard error (SE), odds ratio, 
and case and control sample sizes to recapitulate case and control AFs. 

*Results*

In simulations and real data, we find estimating case and control AFs using the whole sample AF 
is highly accurate across all settings. When applying this method to the Pan-UK Biobank we find 
high concordance with the known AFs. Conversely, we find that in Pan-UK Biobank and simulations 
including covariates, deriving case and control AFs from the SE underestimates the minor AF (MAF)
for higher MAFs. To enable estimation of case and control AFs using SE, we use gnomAD v3.1.2 AFs as 
a proxy for true AFs to estimate and correct for bias fit. While estimating the case control AF 
using the whole sample AF is preferred due to its high accuracy, estimating from the SE can be used
more broadly since the SE can be calculated from the p-value and beta estimate, which are more 
commonly provided. Both methods expand the utility of publicly available genetic summary statistics, 
can be used when different statistics are reported, and promote the reusability of genomic data.  


<br><br>
This document introduces you to the functions in CCAFE and gives small examples of how they 
can be used with summary statistics.

<br><br>
Find more details and documentation on GitHub: (https://wolffha.github.io/CCAFE_documentation/)

<br><br>
To jump forward to individual function specifics:

[**CaseControl_AF**](#CaseControl_AF) -- [fast forward to example](#a-quick-demo-of-CaseControl_AF)

[**CaseControl_SE**](#CaseControl_SE) -- [fast forward to example](#a-quick-demo-of-CaseControl_SE)
<br><br><br>

To view details on using GWAS VCF formatted data and Bioconductor packages to prepare your data for
use in CCAFE:

[Fast forward to demo](#VCF)


# Installation

To install this package from BioConductor:

```{r, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("CCAFE")
```

To install this package from GitHub:

```{r, eval=FALSE}
if(!require(devtools, quietly = TRUE)) {
  install.packages("devtools")
}

devtools::install_github("https://github.com/wolffha/CCAFE")
```

## Overview of CCAFE Functions

CCAFE contains two functions: 

* **CaseControl_AF()**
* **CaseControl_SE()**

The two functions are both used to reconstruct case and control allele frequencies (AFs) from 
genetic summary statistics. 

The user will select which function to use based on the available summary statistics.

**CaseControl_AF()** has the following required parameters:

* Number of cases
* Number of controls
* Odds Ratio (OR) or beta coefficient
* **AF** for the whole sample (cases and controls combined)

**CaseControl_SE()** has the follwoing required parameters:

* Number of cases
* Number of controls
* Odds Ratio (OR) or beta coefficient
* **SE** of the log(OR) for each variant

*Code adapted from ReACt GroupFreq function available here: (https://github.com/Paschou-Lab/ReAct/blob/main/GrpPRS_src/CountConstruct.c)*

# CaseControl_AF {#CaseControl_AF}

## CaseControl_AF() input

* **data**: a dataframe with each row being a variant and columns for total AF and OR
* **N_case**: an integer for the number of case samples
* **N_control**: an integer for the number of control samples
* **OR_colname**: a string containing the exact column name in *data* with the odds ratios
* **AF_total_colname**: a string containing the exact column name in *data* with the total AFs

## CaseControl_AF() output

Returns the original *data* dataframe with two additional columns: 

* AF_case 
* AF_control

The number of rows is equal to the number of variants.

# CaseControl_SE {#CaseControl_SE}

## CaseControl_SE() input

CaseControl_SE has the following required inputs:

* **data**: a dataframe where each row is a variant and columns for the OR, SE, chromosome, and 
position
* **N_case**: an integer for the number of case samples
* **N_control**: an integer for the number of control samples
* **OR_colname**: a string containing the exact column name in *data* with the odds ratios
* **SE_colname**: a string containing the exact column name in *data* with the standard errors
* **position_colname**: a string containing the exact column name in *data* with the positions of 
the variants
* **chromosome_colname**: a string containing the exact column name in *data* with the chromosome 
of the variants. Note, sex chromosomes can be either characters ('X', 'x', 'Y', 'y') or numeric 
where X=23 and Y=24
* **sex_chromosomes**: boolean, TRUE if variants from sex chromosome(s) are included in the dataset
* **do_correction**: boolean, TRUE if data is provided to correct the estimates using proxy MAFs
* **remove_sex_chromosomes**: boolean, TRUE if variants on sex chromosomes should be removed. This 
is only necessary if *sex_chromosomes* == TRUE and the number of XX/XY individuals per case and 
control sample is NOT known

CaseControl_SE has the following optional inputs: 

If *sex_chromosomes* == TRUE and *remove_sex_chromosomes* == FALSE, the following inputs are required:

* **N_XX_case**: the number of XX chromosome case individuals
* **N_XX_control**: the number of XX chromosome control individuals
* **N_XY_case**: the number of XY chromosome case individuals
* **N_XY_control**: the number of XY chromosome control individuals

If *do_correction* == TRUE, then data must be provided that includes harmonized data with proxy MAFs

* **correction_data**: a dataframe with the following EXACT column names: CHR, POS, proxy_MAF, 
containing data for variants harmonized between the observed and proxy datasets

## CaseControl_SE() output

Returns the *data* dataframe with three additional columns: 

* MAF_case
* MAF_control 
* MAF_total 

Each columns contains the estimated MAF in the cases, controls, and whole sample, respectively. 
The number of rows is equal to the number of variants

If *do_correction* == TRUE, three additional columns are included in the dataframe:

* MAF_case_adj
* MAF_control_adj 
* MAF_total_adj

Containing MAFs adjusted using the the proxy MAFs to model the expected bias.

*NOTE:* This method assumes we are estimating the minor allele frequency (MAF). The minor allele 
may or may not be the effect variant used to calculate AF for the GWAS summary statistics. For 
additional details see vignette titled "CCAFE Extra Details"

# Examples

These examples use the provided sample data which is a subset of 500 variants from chromosome 1 
of the Pan-UKBB diabetes GWAS in non-Finnish European individuals.

```{r, echo=TRUE,message=FALSE}
library(CCAFE)
library(tidyverse)
```

## A quick demo of CaseControl_AF() {#a-quick-demo-of-CaseControl_AF}

Here is a quick demonstration of using the CaseControl_AF function in the CCAFE package using the sample data.
This code block loads the library. It then loads the sample data and ensures that it is in the required format (dataframe). 
It then runs the methods using the required parameters (data, N_case, N_control, OR_colname, AF_total_colname) and prints out
the first few lines of the resulting returned dataframe. 

```{r CaseControl_AF example}
# load the data
data("sampleDat")
sampleDat <- as.data.frame(sampleDat)

results_af <- CaseControl_AF(data = sampleDat,
                             N_case = 16550,
                             N_control = 403923,
                             OR_colname = "OR",
                             AF_total_colname = "true_maf_pop")

head(results_af)
```
We can then plot the estimates from this example using the provided known AFs. 

```{r, eval=TRUE}
# plot the results
# first need minor allele frequency to compare to true_maf_case
results_af$MAF_case <- sapply(results_af$AF_case, function(x)
  ifelse(x > 0.5, 1-x, x))
results_af <- as.data.frame(results_af)

ggplot(results_af, aes(x = true_maf_case, y = MAF_case)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  ggtitle("Example CaseControl_AF()")
```

## A quick demo of CaseControl_SE() {#a-quick-demo-of-CaseControl_SE}

Here is a quick demonstration of using the CaseControl_SE function in the CCAFE package using the 
sample data.
This code block loads the library. It then loads the sample data and ensures that it is in the 
required format (dataframe). 
It then runs the methods using the required parameters (data, N_case, N_control, OR_colname, 
SE_colname, chromosome_colname, position_colname, do_correction, sex_chromosomes) and prints out the
first few lines of the resulting returned dataframe. 

```{r CaseControl_SE example - no correction}
# load the data
data("sampleDat")

# always ensure data is a dataframe before inputting to CCAFE methods
sampleDat <- as.data.frame(sampleDat)

# First run without correction
results_se_noCorr <- CaseControl_SE(data = sampleDat,
                                    N_case = 16550,
                                    N_control = 403923,
                                    OR_colname = "OR",
                                    SE_colname = "SE",
                                    chromosome_colname = "CHR",
                                    position_colname = "POS",
                                    do_correction = FALSE,
                                    sex_chromosomes = FALSE)
head(results_se_noCorr)
```

We can then plot the estimates from this example using the provided known AFs. 

```{r, eval=TRUE}
# plot the results
ggplot(results_se_noCorr, aes(x = true_maf_case, y = MAF_case)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  coord_cartesian(ylim = c(0,.5)) +
  ggtitle("Example CaseControl_SE() No Correction")
```

Here is a quick demonstration of using the CaseControl_SE function in the CCAFE package using the 
sample data, but also performing the provided bias correction.
This code block loads the library. It then loads the sample data and ensures that it is in the 
required format (dataframe). 
It then runs the methods using the required parameters (data, N_case, N_control, OR_colname, 
SE_colname, chromosome_colname, position_colname, do_correction, sex_chromosomes) and prints out the
first few lines of the resulting returned dataframe. 

```{r CaseControl_SE example - correction}
# load the data
data("sampleDat")
# always ensure data is a dataframe before inputting to CCAFE methods
sampleDat <- as.data.frame(sampleDat)

corr_data <- data.frame(CHR = sampleDat$CHR,
                        POS = sampleDat$POS,
                        proxy_MAF = sampleDat$gnomad_maf)

# now run with correction
results_se_corr <- CaseControl_SE(data = sampleDat,
                                  N_case = 16550,
                                  N_control = 403923,
                                  OR_colname = "OR",
                                  SE_colname = "SE",
                                  chromosome_colname = "CHR",
                                  position_colname = "POS",
                                  do_correction = TRUE,
                                  correction_data = corr_data,
                                  sex_chromosomes = FALSE)

head(results_se_corr)
```
Again, we can plot the estimates (corrected) and compare them to the provided known AFs and 
see reduced bias. 

```{r, eval=TRUE}
# plot the results
ggplot(results_se_corr, aes(x = true_maf_case, y = MAF_case_adj)) +
  geom_point() +
  geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "red") +
  theme_bw() +
  ggtitle("Example CaseControl_SE() With Correction")
```

# Integration with Bioconductor 

## Tabular data

This software package aims to increase the utility of existing, publicly available genomic summary 
data from genome-wide association studies (GWAS). Summary statistics are still widely underutilized,
despite their wide availability. As such, Bioconductor packages with methods specifically designed
to analyze these statistics are still being developed. 

The most commonly used repository of these summary statistics is the NHGRI GWAS Catalog (https://www.ebi.ac.uk/gwas/). From the GWAS catalog, a user can download full summary statistics 
through the FTP sites. The majority of these summary statistics are zipped plain text (usually space
or tab delimited) files. The GWAS Catalog itself has recommended a tabular, plain-text format as its
recommended GWAS summary statistic format. As such, this tabular data is most simply read into R as 
a dataframe structure. 

## VCF data {#VCF}

While the many users will download the GWAS files and load them into R, there are other ways to 
obtain GWAS summary statistics, including through the IEU Open GWAS Project
(https://gwas.mrcieu.ac.uk/). The IEU Open GWAS Project instead uses the GWAS VCF format
(https://doi.org/10.1101/2020.05.29.115824) to store GWAS summary statistics and ensure alignment 
to the hg19 reference genome. We can use an assortment of Bioconductor packages and data structures 
to work with GWAS VCF data. 

First we will make use of the Bioconductor package *VariantAnnotation* to handle VCF data

```{r, eval=FALSE}
BiocManager::install("VariantAnnotation")
```

```{r}
suppressWarnings(suppressPackageStartupMessages({
  library(VariantAnnotation)
  library(tidyverse)
}))
```

We will use the VCF from this GWAS of Type 2 Diabetes https://doi.org/10.1038/s41588-018-0084-1.
A subset of 10,000 variants has been stored within the package and can be loaded. 

```{r}
data("vcf_sample")
```

We can  see the name of the GWAS dataset (refers to the IEU GWAS database ID name)

```{r}
samples(header(vcf_sample))
```
To understand what the data format, we can look view the VCF object

```{r}
vcf_sample
```

The columns that we want for use in CCAFE are:

From the rowRanges object:

* seqnames (chromosome)
* ranges (position)
* REF
* ALT
* RSID

From the geno object: 

* ES (effect size of ALT)
* SE
* AF (allele frequency of ALT)

We then want to convert this VCF formatted dataset into a simple dataframe to use with CCAFE.

```{r}
# first we will get the info from GRanges object (position, RSID)
meta <- as.data.frame(ranges(vcf_sample))
meta <- meta[,c(1, 4)]
colnames(meta) <- c("Position", "RSID")
# get the chromosome (as a)
meta$Chromosome <- as.vector(seqnames(rowRanges(vcf_sample)))

# now we can also get the meta data (REF, ALT) from the GRanges object
meta <- cbind(meta, mcols(vcf_sample)[,c(2,3)])
rownames(meta) <- seq(1, nrow(meta))

# now we will get the info from the geno object
geno_dat <- data.frame(
  beta = unlist(geno(vcf_sample)$ES),
  SE = unlist(geno(vcf_sample)$SE),
  AF = unlist(geno(vcf_sample)$AF)
)

df_data <- cbind(meta, geno_dat)
head(df_data)
```

For CCAFE we will use OR, so let's create this column by exponentiating the effect estimate

```{r}
df_data$OR <- exp(df_data$beta)
```

Let's now apply the CCAFE methods to get case and control specific AFs

First, since we have total AF, we'll use CaseControl_AF

```{r}
df_data <- CaseControl_AF(data = df_data,
                              N_case = 48286,
                              N_control = 250671,
                              OR_colname = "OR",
                              AF_total_colname = "AF")
head(df_data)
```

We can look at a comparison of the AFs 

Create a dataframe to plot

```{r}
plotdata_AF <- df_data %>% dplyr::select(AF, AF_case, AF_control)
plotdata_AF$AF_total <- (plotdata_AF$AF_case*48286 + plotdata_AF$AF_control*250671)/(298857)
colnames(plotdata_AF)[1] <- "AF(known)"
plotdata_AF_long <- pivot_longer(plotdata_AF, cols = colnames(plotdata_AF))
```

Plot the results

```{r}
ggplot(plotdata_AF_long, aes(x = value, fill = name)) +
  geom_boxplot(alpha = 0.5, outliers = FALSE) +
  facet_wrap(~name, nrow = 4) +
  theme_bw()
```

These case and control AFs can now be used in further downstream analyses. For example, if the data
was harmonized with a reference panel, then Summix2 
(https://www.bioconductor.org/packages/release/bioc/html/Summix.html) could be used to estimate 
substructure in the GWAS sample (total sample, cases, or controls). The AFs could also be used in 
case-case GWAS (see https://wolffha.github.io/CCAFE_documentation/articles/CCAFE_CCGWAS.html) or 
certain meta-analysis software (e.g. https://doi.org/10.1038/s41598-022-12185-6). 

### VCF Conversion Function

For ease of use of CCAFE with VCF files, we have included a function 'CCAFE_convertVCF()'. 

It performs the steps outlined above, assuming the user has loaded their VCF into R. 

```{r CCAFE_convertVCF Demo}
df_data_2 <- CCAFE_convertVCF(vcf_sample)
```

```{r}
df_data_2 <- CaseControl_AF(data = df_data_2,
                              N_case = 48286,
                              N_control = 250671,
                              OR_colname = "OR",
                              AF_total_colname = "AF")
head(df_data_2)
```

# Session Information

```{r}
sessionInfo()
```

---
title: "CCAFE CC-GWAS Example"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup1}
library(CCAFE)
```

---
title: "CaseControlAF_ccGWAS"
author: "Hayley Stoneman"
date: "2024-09-12"
output: word_document
---

# Introduction and motivation

In this vignette, I will demonstrate the workflow for using the R package *CaseControlAF* and 
perform secondary analyses using the output case and control AFs. Here, the secondary analysis
will be a case-case GWAS (CC-GWAS). Additional details of CC-GWAS can be found in Chapter I: 
INTRODUCTION. 

## Brief overview of CC-GWAS 

Briefly, CC-GWAS identifies genetic variants that have different AFs between two samples of cases 
with related (but different) traits or diseases. This could provide insight as to why a person might
develop one trait instead of a closely related second trait. CC-GWAS was first introduced by Peyrot 
and Price in 2021. A second alternative framework to perform CC-GWAS was proposed by Yang *et al.* 
in 2022. Yang *et al* provided C-based software to perform this analysis as part of a larger 
software package called *ReACt*. I have adapted their code and framework and implemented it in R. 

In this framework a logistic regression is run using only the AFs from the two case samples. 
However, it is important to differentiate the effects from trait-trait differences and population
stratification. In this logistic regression of the two case samples, the effect sizes $\beta_1^{case}$ 
will include real genetic differences between trait 1 and trait 2 ($\beta_g$) as well as differences 
due to population stratification ($\beta_s$). We assume that these two effects are independent 
such that:

$$ \beta_1^{case} = \beta_g + \beta_s $$
To isolate the true trait-trait genetic differences, a regression between the two controls samples
is run, to get an estimate of the stratification effect $\beta_s$. Note that we also get a standard
error for the estimate of stratification, and this can be used to remove variants for which the 
stratification effect cannot be reliably estimated. 

The genetic effect from the trait-trait difference can then be obtained: 

$$ \beta_g = \beta_1^{case} - \beta_s $$
These case and control logistic regressions are carried out using the case and control genotype
counts and the binary phenotype. Here I will compare the known case and control genotype counts 
(included in the summary statistics), those estimated using the SE (CaseControl_SE) and those 
estimated using total AF (CaseControl_AF). 

# Datasets

I will be using GWAS summary statistics from the All of Us Research Program All by All (AllxAll) 
tables. The All of Us Research Program recently made available to the Controlled Tier Access
researchers the results from GWAS for thousands of phenotypes from the ~250,000 participants who
have whole genome sequence data available. The AllxAll tables include the results of association 
testing for ~3,4000 phenotypes including physical measurements, lab measurements, phecodes, and 
electronic health record (EHR) sourced drugs and medications. The data is available in Hail
Matrix Tables with the summary statistics by individual ancestry or following meta-analysis. Find
more information about the AllxAll results [here](https://support.researchallofus.org/hc/en-us/articles/27049847988884-Overview-of-the-All-by-All-tables-available-on-the-All-of-Us-Researcher-Workbench) 

In the original CC-GWAS paper by Peyrot and Price 2021, psychiatric traits were compared, including 
Schizophrenia (SZ) and Bipolar Disease (BP). Yang *et al* 2022 also compared these two traits.

In All of Us, GWAS was run for SZ (Phecode 295.1), with 584 cases and 98,402 controls. GWAS for 
BP (Phecode 296.1) was run with 2,865 cases and 92,758 controls. These two Phecodes exclude other
psychiatric trait ICD codes in their case definitions, so there should be no overlap between case
samples in the two traits. Here I will query the association summary statistics for these two 
GWAS, including the total AF, true case and control AFs, beta/OR, and SE of the log(OR). 

## Obtain datasets from All of Us Researcher Workbench

Here I will show the code used to obtain the summary statistics from these GWAS from the AllxAll
tables. I obtain the case and control sample size, the case and control AFs, the total AF and allele
count (AC), and the variant ID, beta, and SE values for each variant. Note that this code must be
run through the Researcher Workbench after obtaining Controlled Tier Access. I am only showing the 
code used to obtain the BP data set (phenoname = 296.1), though the steps are identical just with a 
different phenotype name for SZ (phenoname = 295.1). The Researcher Workbench uses Jupyter Notebooks
running Python 3. 

```{python setup, eval=F}
# setting up packages 
from datetime import datetime 
import os 
import gc 
 
#for data handling and storage 
import subprocess 
import numpy as np 
import pandas as pd 
 
start = datetime.now() 
 
# loading bokeh to show hail plots 
from bokeh.resources import INLINE 
import bokeh.io 
from bokeh import * 
bokeh.io.output_notebook(INLINE) 
from bokeh.io import show, output_notebook 
 
# Google cloud resources 
from google.cloud import storage 
 
# initializing hail 
# setting hail tmp directory to WORKSPACE_BUCKET. This prevents hail to use HDFS and a risk of 
# running out of space 
# if there is not enough space on non-preemptible workers 
# this may slow down I/O of large files but prevents scripts from crushing 
import hail as hl 
hl.init(default_reference = "GRCh38") 

bucket = os.getenv('WORKSPACE_BUCKET') 
```

Read in Hail matrix table for the ACAF (single variant) data. I am using the EUR only data due to
the small sample size. 

*SZ*

* Cases EUR = 584 
* Controls EUR = 98,402

*BP*

* Cases EUR = 2,865 cases
* Controls EUR = 92,758

```{python, eval=F}
all_EUR = hl.read_matrix_table(“gs://fc-aou-datasets-controlled/AllxAll/v1/mt/EUR_ACAF_results.mt”) 
```

Subset the data to only the desired phenotype by phenoname
```{python, eval=F}
Pheno = “296.1” 

Bp_EUR = all_EUR.filter_cols(all_EUR.phenoname == Pheno) 
Bp_EUR = Bp_EUR.entries() 

# View the data set 
Bp_EUR.show() 
```

Subset to only the desired columns
```{python, eval=F}
Bp_EUR_sub = bp_EUR.select('variant_id', 'n_cases', 'n_controls', 'BETA', 'SE', 'AF_case', 
'AF_ctrl', 'AF_Allele2', 'AC_Allele2') 
```

Export the data to your Google bucket
```{python, eval=F}
Bp_path = f’{bucket}/data/pheno_ENTERNAMEHERE.tsv.bgz’ 

Bp_EUR_sub.export(Bp_path) 
```

# Source code

Here is the source code used to perform the CC-GWAS and generate plots. The CC-GWAS code and helper
functions have been adapted from the ReACt software, which was original written in C. The code can
be found [here](https://github.com/Paschou-Lab/ReAct).

```{r source_code, eval=F}
# Helper functions

# Function to calculate the p-value of a wald statistic assuming normal distribution
calc_waldP <- function(beta, SE) {
  z <- abs(beta/SE)
  p_val <- 2*(1-pnorm(z))
  return(p_val)
}

# function to calculate OR, SE, and p-value using a 2x2 table
# assumed to be using the AF of trait 1, AF of trait 2, and sample sizes
# DefFac is used to scale the allele counts calculated based on sample overlap
calc_ORStat <- function(AF1, N1, AF2, N2, DefFac) {
  # first calculate the 2x2 tables of allele counts
  a = AF1 * 2 * N1 * DefFac[1]
  b = (1-AF1) * 2 * N1 * DefFac[1]
  c = AF2 * 2 * N2 * DefFac[2]
  d = (1-AF2) * 2 * N2 * DefFac[2]
  
  OR <- (a*d)/(b*c)
  SE <- sqrt(1/a + 1/b + 1/c + 1/d)
  p <- calc_waldP(log(OR), SE)
  
  return(data.frame(OR = OR, SE = SE, p = p))
}

get_z <- function(OR1, OR2, SE1, SE2, Zthres) {
  N12 = 0 # number of times Z1 and Z2 both non zero
  SumZ12 = 0 # sum of Z1*Z2
  SqrSumZ1 = 0 # sum of Z1^2
  SqrSumZ2 = 0 # sum of Z2^2
  
  for(i in 1:length(OR1)) {
    # set z to Log(OR)/SE as long as OR != 0, else set it to 0
    z1 = ifelse(OR1[i]*SE1[i] != 0, log(OR1[i])/SE1[i], 0.0)
    z2 = ifelse(OR2[i]*SE2[i] != 0, log(OR2[i])/SE2[i], 0.0)
    
    # if |z| > zthreshold set to zero
    z1 = ifelse(z1 > Zthres, 0.0, z1)
    z2 = ifelse(z2 > Zthres, 0.0, z2)
    
    # if product is not zero then update z square values
    if(z1*z2 != 0) {
      N12 <- N12 + 1
      SumZ12 <- SumZ12 + z1*z2
      SqrSumZ1 <- SqrSumZ1 + z1^2
      SqrSumZ2 <- SqrSumZ2 + z2^2
    }
  }
  return(c(N12, SumZ12, SqrSumZ1, SqrSumZ2))
}

# helper function to calculate log-likelihood for a bivariate normal distribution given the correlation coefficient rho and total smaple size N
calc_logLikelihood <- function(x2, y2, xy, rho, N, Zthres) {
  l_0 <- - N/2 * log(1 - rho^2) - (x2 - 2*xy*rho + y2)/(2 * (1 - rho^2))
  l_1 <- N*log(pbivnorm(-Zthres, -Zthres, rho) +
                 pbivnorm(Zthres, Zthres, rho) +
                 pbivnorm(Zthres, -Zthres, rho) +
                 pbivnorm(-Zthres, Zthres, rho))
  return(l_0 - l_1)
}

calc_corrR <- function(OR1, OR2, SE1, SE2, Zthres) {
  bestRho = 0.0
  Zs <- get_z(OR1, OR2, SE1, SE2, Zthres)
  maxLogL = calc_logLikelihood(x2 = Zs[3], y2 = Zs[4], xy = Zs[2], rho = bestRho, N = Zs[1],
                               Zthres = Zthres)
  for(rho in seq(from = 0.001, to = .999, by = .001)) {
    tmpLogL = calc_logLikelihood(x2 = Zs[3], y2 = Zs[4], xy = Zs[2], rho = rho, N = Zs[1],
                                 Zthres = Zthres)
    if(tmpLogL > maxLogL) {
      maxLogL = tmpLogL
      bestRho = rho
    }
  }
  return(bestRho)
}

# main helper function for CC-GWAS
# uses the case and control AF for each of the two traits as well as the case and control
# sample sizes of each trait
calc_ccgwas <- function(trait1_AF_case, trait2_AF_case, trait1_N_case, trait2_N_case,
                        trait1_AF_control, trait2_AF_control, trait1_N_control, trait2_N_control,
                        N_caseInCase = NA, N_controlInControl = NA, N_caseInControl = NA,
                        Zthreshold = 0.0, trait1_OR, trait2_OR, trait1_SE, trait2_SE) {
  
  # first define the sample overlap matrices
  if(length(N_caseInCase) == 0) {
    N_caseInCase <- matrix(data = c(trait1_N_case + trait2_N_case, 0, 0, trait1_N_case + trait2_N_case), nrow = 2)
  }
  if(length(N_controlInControl) == 0) {
    N_controlInControl <- matrix(data = c(trait1_N_control + trait2_N_control, 0, 0, 
                                          trait1_N_control + trait2_N_control), nrow = 2)
  }
  if(length(N_caseInControl) == 0) {
    N_caseInControl <- matrix(data = c(0, 0, 0, 0), nrow = 2)
  }
  
  # correct for sample overlap  by calculating correlation
  if(Zthreshold > 0.0) {
    CorrR = calc_corrR(OR1 = trait1_OR, OR2 = trait2_OR, SE1 = trait1_SE,
                       SE2 = trait2_SE, Zthres = Zthreshold)
    print(paste0("Est. sample overlap: ", CorrR))
    
    # update sample overlap matrices
    N_caseInCase[1,2] = sqrt(N_caseInCase[1,1]*N_caseInCase[2,2])*CorrR
    N_caseInCase[2,1] = sqrt(N_caseInCase[1,1]*N_caseInCase[2,2])*CorrR
    N_controlInControl[1,2] = sqrt(N_controlInControl[1,1]*N_controlInControl[2,2])*CorrR
    N_controlInControl[2,1] = sqrt(N_controlInControl[1,1]*N_controlInControl[2,2])*CorrR
  }
  
  # now we can calculate the default factor for trait 1 and 2 in cases and controls
  DefCase <- c((N_caseInCase[1,1]/(N_caseInCase[1,1] + N_caseInCase[1,2])),
               (N_caseInCase[2,2]/(N_caseInCase[2,2] + N_caseInCase[2,1])))
  DefControl <- c((N_controlInControl[1,1]/(N_controlInControl[1,1] + N_controlInControl[1,2])),
                  (N_controlInControl[2,2]/(N_controlInControl[2,2] + N_controlInControl[2,1])))
  
  # now we calculate the OR, SE, and p for cases and controls separately
  # controls are used for analysis of stratification 
  # overall final CC-GWAS result OR is exp(log(OR_case) - log(OR_control))
  OR_stat_control <- calc_ORStat(trait1_AF_control, trait1_N_control, trait2_AF_control, trait2_N_control, DefControl)
  OR_stat_case <- calc_ORStat(trait1_AF_case, trait1_N_case, trait2_AF_case, trait2_N_case, DefCase)
  
  OR <- exp(log(OR_stat_case$OR)-log(OR_stat_control$OR))
  SE <- OR_stat_case$SE
  p <- calc_waldP(log(OR), SE)
  
  return(data.frame(OR = OR, SE = SE, p = p,
                    OR_control = OR_stat_control$OR,
                    SE_control = OR_stat_control$SE))
}


#' @title CC_GWAS
#' @description
#' This is a function to run a case-case GWAS using the framework introduced in the ReACt package. The method will first filter out any SNPs with NA in the case or control AF. It will then remove any SNPs that are not present in both datasets. Finally, it will remove duplicate SNPs, by defualt keeping the first record. Then the CC-GWAS will be run. 
#' 
#' @param trait1_data a dataframe containing the chromosome, position, case AF, and control AF for each SNP for trait 1
#' @param trait2_data a dataframe containing the chromosome, position, case AF, and control AF for each SNP for trait 2
#' @param trait1_N_case the number of cases in trait 1 dataset
#' @param trait1_N_control the number of controls in trait 1 dataset
#' @param trait2_N_case the number of cases in trait 2 dataset
#' @param trait2_N_control the number of controls in trait 2 dataset
#' @param AF_case_colname a string with the name for the column containing the case AF in both trait 1 and trait 2 datasets; default: "AF_case"
#' @param AF_control_colname a string with the name for the column containing the control AF in both trait 1 and trait 2 datasets; default: "AF_control"
#' @param chromosome_colname a string with the name for the column containing the SNP chromosome number in both trait 1 and trait 2 datasets; default: "chr"
#' @param position_colname a string with the name for the column containing the SNP position in both trait 1 and trait 2 datasets; default: "pos"
#' @param id_colname a string containing the id for each SNP in both trait 1 and trait 2 datasets; default: NA. If NA, then an ID column will be added and used for data filtering
#' @param OR_colname a string with the name for the column with the OR from the GWAS summary statistics for each trait
#' @param SE_colname a string with the name for the column with the SE from the GWAS summary statistics for each trait
#' @param N_caseInCase a 2x2 matrix where the upper left cell (a) is the number of cases in study 1 and the lower right cell (d) is the number of cases in study 2. Cells b and c are the number of overlapping cases; Default = NA - if left as NA algorithm will assume no sample overlap
#' @param N_controlInControl a 2x2 matrix where the upper left cell (a) is the number of controls in study 1 and the lower right cell (d) is the number of controls in study 2. Cells b and c are the number of overlapping controls; Default = NA - if left as NA algorithm will assume no sample overlap
#' @param N_caseInControl a 2x2 matrix of the number of cases in the control sample. The upper right cell (b) represents the number of cases from trait 1 that are present as controls for trait 2. The lower left cell (c) is the number of cases from trait 2 that are included as controls for trait 1. Cells a and d should be 0; Default = NA - if left as NA algorithm will assume no sample overlap
#' @param Zthreshold value for the z-statistic threshold. If Zthreshold = 0 then will not perform correction for sample overlap (In order to run correction for sample overlap, OR and SE in each trait's data is required). Yang et al suggest using a threshold of z=1.0; however, this will overlap user input sample sizes. If exact sample sizes and overlap is known, this parameter should be left at 0. 
#' @note
#'  trait1_data and trait2_data need to have the same column names for chromosome, position, OR, SE, and case and control AFs.
#' @return returns a dataframe with the original data (where each dataset's original columns with have .trait1 or .trait2 appended to its column name) and three additional columns for the CC-GWAS results: OR, SE, p
#' equal to the number of variants
#' @import tidyverse
#' @import pbivnorm
#' @export 
CC_GWAS <- function(trait1_data, trait2_data, 
                    AF_case_colname = "AF_case", AF_control_colname = "AF_control",
                    trait1_N_case, trait1_N_control, trait2_N_case, trait2_N_control,
                    chromosome_colname = "chr", position_colname = "pos", id_colname = NA,
                    A1_colname = "A1", A2_colname = "A2", rsid_colname = "RSID", OR_colname = "OR",
                    SE_colname = "SE",
                    N_caseInCase = NA, N_controlInControl = NA, N_caseInControl = NA,
                    Zthreshold = 0.0) {
  require(tidyverse)
  require(pbivnorm)
  # First need to make sure there are no SNPs with NA is the case or control AFs
  NA_case_1 = length(which(is.na(trait1_data[,c(AF_case_colname)])))
  NA_case_2 = length(which(is.na(trait2_data[,c(AF_case_colname)])))
  NA_control_1 = length(which(is.na(trait1_data[,c(AF_control_colname)])))
  NA_control_2 = length(which(is.na(trait2_data[,c(AF_control_colname)])))
  
  NA_rm1 = 0
  NA_rm2 = 0
  
  if(NA_case_1 > 0 | NA_control_1 > 0) {
    trait1_data_filt <- trait1_data[!is.na(trait1_data[,c(AF_case_colname)]) &
                                      !is.na(trait1_data[,c(AF_control_colname)]),]
    NA_rm1 = nrow(trait1_data)-nrow(trait1_data_filt)
    trait1_data <- trait1_data_filt
    rm(trait1_data_filt)
  }
  if(NA_case_2 > 0 | NA_control_2 > 0) {
    trait2_data_filt <- trait2_data[!is.na(trait2_data[,c(AF_case_colname)]) &
                                      !is.na(trait2_data[,c(AF_control_colname)]),]
    NA_rm2 = nrow(trait2_data)-nrow(trait2_data_filt)
    trait2_data <- trait2_data_filt
    rm(trait2_data_filt)
  }
  rm(NA_case_1)
  rm(NA_case_2)
  rm(NA_control_1)
  rm(NA_control_2)
  
  # now make sure that both datasets have the same variants
  if(is.na(id_colname)) {
    trait1_data$id <- paste0(trait1_data[,c(chromosome_colname)], ":", 
                             trait1_data[,c(position_colname)])
    trait2_data$id <- paste0(trait2_data[,c(chromosome_colname)], ":", 
                             trait2_data[,c(position_colname)])
    if(nrow(trait1_data) < nrow(trait2_data)) {
      overlap <- trait1_data$id[which(trait1_data$id %in% trait2_data$id)]
    } else {
      overlap <- trait2_data$id[which(trait2_data$id %in% trait1_data$id)]
    }
    
    id_colname = "id"
  } else {
    colnames(trait1_data)[which(colnames(trait1_data) == id_colname)] = "id"
    colnames(trait2_data)[which(colnames(trait2_data) == id_colname)] = "id"
    if(nrow(trait1_data) < nrow(trait2_data)) {
      overlap <- trait1_data$id[which(trait1_data$id %in% trait2_data$id)]
    } else {
      overlap <- trait2_data$id[which(trait2_data$id %in% trait1_data$id)]
    }
  }
  overlap_rm1 = nrow(trait1_data)-length(overlap)
  overlap_rm2 = nrow(trait2_data)-length(overlap)
  trait1_data <- trait1_data %>% filter(id %in% overlap)
  trait2_data <- trait2_data %>% filter(id %in% overlap)
  
  # now keep only distinct 
  trait1_data_filt <- trait1_data %>% distinct(id, .keep_all = T)
  trait2_data_filt <- trait2_data %>% distinct(id, .keep_all = T)
  distinct_rm1 = nrow(trait1_data) - nrow(trait1_data_filt)
  distinct_rm2 = nrow(trait2_data) - nrow(trait2_data_filt)
  trait1_data = trait1_data_filt
  rm(trait1_data_filt)
  trait2_data = trait2_data_filt
  rm(trait2_data_filt)
  
  nRemoved <- data.frame(dataset = c(1, 2),
                         NA_removed = c(NA_rm1, NA_rm1),
                         overlap_removed = c(overlap_rm1, overlap_rm2),
                         distinct_removed = c(distinct_rm1, distinct_rm2))
  print(("Removed the following number of variants due to NAs in AFs, SNPs not in both datasets, or duplicate SNPs:"))
  print(nRemoved)
  
  # rename columns so I can use dplyr to select desired ones for output
  colnames(trait1_data)[which(colnames(trait1_data) == chromosome_colname)] = "chr"
  colnames(trait1_data)[which(colnames(trait1_data) == position_colname)] = "pos"
  colnames(trait1_data)[which(colnames(trait1_data) == rsid_colname)] = "RSID"
  colnames(trait2_data)[which(colnames(trait2_data) == chromosome_colname)] = "chr"
  colnames(trait2_data)[which(colnames(trait2_data) == position_colname)] = "pos"
  colnames(trait2_data)[which(colnames(trait2_data) == rsid_colname)] = "RSID"
  
  if(AF_case_colname != "AF_case") {
    trait1_data$AF_case <- trait1_data[,c(AF_case_colname)]
    trait2_data$AF_case <- trait2_data[,c(AF_case_colname)]
  }
  if(AF_control_colname != "AF_control") {
    trait1_data$AF_control <- trait1_data[,c(AF_control_colname)]
    trait2_data$AF_control <- trait2_data[,c(AF_control_colname)]
  }
  if(A1_colname != "A1") {
    trait1_data$A1 <- trait1_data[,c(A1_colname)]
    trait2_data$A1 <- trait2_data[,c(A1_colname)]
  }
  if(A2_colname != "A2") {
    trait1_data$A2 <- trait1_data[,c(A2_colname)]
    trait2_data$A2 <- trait2_data[,c(A2_colname)]
  }
  if(OR_colname != "OR") {
    trait1_data$OR <- trait1_data[,c(OR_colname)]
    trait2_data$OR <- trait2_data[,c(OR_colname)]
  }
  if(SE_colname != "SE") {
    trait1_data$SE <- trait1_data[,c(SE_colname)]
    trait2_data$SE <- trait2_data[,c(SE_colname)]
  }
  
  # make sure trait1 and trait2 are in the same order 
  if(nrow(trait1_data) != nrow(trait2_data)) {
    stop("Number of SNPs doesn't match")
  }
  combined <- trait1_data %>% inner_join(trait2_data, by = c("id" = "id"),
                                         suffix = c(".trait1", ".trait2"))
  # make sure alleles match (if they don't switch allele in trait 2 to match trait 1)
  combined[which(combined$A1.trait1 != combined$A1.trait2), ]$AF_case.trait2 <- 1-combined[which(combined$A1.trait1 != combined$A1.trait2), ]$AF_case.trait2
  combined[which(combined$A1.trait1 != combined$A1.trait2), ]$AF_control.trait2 <- 1-combined[which(combined$A1.trait1 != combined$A1.trait2), ]$AF_control.trait2
  
  gc()
  
  combined <- as.data.frame(combined)

  # run the actual CC-GWAS using the cleaned datasets
  results <- calc_ccgwas(trait1_AF_case = combined[,"AF_case.trait1"],
                         trait2_AF_case = combined[,"AF_case.trait2"],
                         trait1_AF_control = combined[,"AF_control.trait1"],
                         trait2_AF_control = combined[,"AF_control.trait2"],
                         trait1_N_case = trait1_N_case,
                         trait2_N_case = trait2_N_case,
                         trait1_N_control = trait1_N_control,
                         trait2_N_control = trait2_N_control,
                         trait1_OR = combined[,"OR.trait1"],
                         trait2_OR = combined[,"OR.trait2"],
                         trait1_SE = combined[,"SE.trait1"],
                         trait2_SE = combined[,"SE.trait2"],
                         Zthreshold = Zthreshold,
                         N_controlInControl = N_controlInControl,
                         N_caseInCase = N_caseInCase,
                         N_caseInControl = N_caseInControl)
  
  meta_cols <- c("chr.trait1", "pos.trait1", "RSID.trait1", "A1.trait1", "A2.trait1",
                 "AF_case.trait1", "AF_case.trait2", "AF_control.trait1",
                 "AF_control.trait2")
  toreturn <- combined[,meta_cols]
  colnames(toreturn)[1:5] <- c("chr", "pos", "RSID", "A1", "A2")
  
  toreturn <- toreturn %>% cbind(results)
  
  return(toreturn)
}

# plotting code
CC_GWAS_plot <- function(results, chromosome_colname = "chr", position_colname = "pos",
                         title = "CC-GWAS") {
  # first make sure the columns have the same names (chr, pos) so I can use tidyverse
  colnames(results)[which(colnames(results) == chromosome_colname)] = "chr"
  colnames(results)[which(colnames(results) == position_colname)] = "pos"
  
  results$chr <- as.numeric(results$chr)
  
  # First need to claculate the cumulatative position of the SNP
  results <- results %>% 
    group_by(chr) %>%
    summarise(chr_length = max(pos)) %>%
    
    # calculate the cumulative position of each chromosome
    mutate(tot = cumsum(as.numeric(chr_length)) - chr_length) %>%
    select(-chr_length) %>%
    
    # add this info to the initial dataset
    left_join(results, ., by=c("chr" = "chr")) %>%
    
    # add cumulative position of each SNP
    arrange(chr, pos) %>%
    mutate(pos_cum = pos + tot)
  
  # need to prepare x-axis so we can show the chromosome number not the SNP position
  axisdf = results %>% group_by(chr) %>%
    summarize(center = (max(pos_cum) + min(pos_cum))/2)
  
  plot <- ggplot(results, aes(x = pos_cum, y = -log10(p))) +
    geom_point(aes(color = as.factor(chr)), alpha = 0.8, size = 1.3) +
    scale_color_manual(values = rep(c("grey40", "black"), 22)) +
    scale_x_continuous(label = axisdf$chr, breaks = axisdf$center) +
    scale_y_continuous(expand = c(0,0)) + 
    geom_hline(yintercept = -log10(0.05/nrow(results)), linetype = "dashed", color = "red") +
    xlab("Chromosome") +
    ylab("-log10(p-value)") +
    ggtitle(title) +
    theme_bw() +
    theme(
      legend.position = "none",
      panel.border = element_blank(),
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank()
    )
  return(plot)
}
```

# CC-GWAS Analysis

```{r setup_server, echo=F, eval=F}
set.seed(1)
setwd("/data001/projects/wolffha/AoU/")

source("/home/wolffha/CC-GWAS/CaseControl_AF.R")
source("/home/wolffha/CC-GWAS/CaseControl_SE.R")
source("/home/wolffha/CC-GWAS/CC_GWAS_20240712.R")

library(tidyverse, lib = "/home/math/wolffha/R/myLibs/")
library(genpwr, lib = "/home/math/wolffha/R/myLibs/")
library(scales, lib = "/home/math/wolffha/R/myLibs/")
library(DescTools, lib = "/home/math/wolffha/R/myLibs/")
library(data.table, lib = "/home/math/wolffha/R/myLibs/")
library(cowplot, lib = "/home/math/wolffha/R/myLibs/")
```

```{r setupR, eval=F}
set.seed(1)
setwd("C:/Users/HayBa/Documents/Graduate School/GeneticPredisposition/CC-GWAS/")

library(tidyverse)
library(scales)
library(DescTools)
library(data.table)
library(CaseControlAF)
```

Load in data and format as needed
```{r data_input, eval=F}
sz <- read.delim("/home/wolffha/CC-GWAS/AoU/295.1/pheno_295.1_EUR.tsv.gz")
bp <- read.delim("/home/wolffha/CC-GWAS/AoU/296.1/pheno_296.1_EUR.tsv.gz")

nCase_sz <- 584
nControl_sz <- 98402
nCase_bp <- 2865
nControl_bp <- 92758

nTot_sz <- nCase_sz + nControl_sz
nTot_bp <- nCase_bp + nControl_bp

# colnames:
# locus | alleles | phenoname | variant+id | n_cases | n_controls | BETA
# SE | AF_case | AF_ctrl | AF_Allele2 | AC_Allele2

colnames(sz) <- c("locus", "alleles", "phenoname", "variant_id", "n_cases", "n_controls",
                  "BETA", "SE", "true_AF_case", "true_AF_control", "AF_Allele2", "AC_Allele2")
colnames(bp) <-  c("locus", "alleles", "phenoname", "variant_id", "n_cases", "n_controls",
                   "BETA", "SE", "true_AF_case", "true_AF_control", "AF_Allele2", "AC_Allele2")

# remove any variants with NAs 
sz <- sz[complete.cases(sz),]
bp <- bp[complete.cases(bp),]

# remove variants with Case and Control MAC < 40 - per AoU requirements
# calculate MAF
sz$maf_case <- sapply(sz$true_AF_case, function(x) ifelse(x > .5, 1-x, x))
sz$maf_control <- sapply(sz$true_AF_control, function(x) ifelse(x > .5, 1-x, x))
bp$maf_case <- sapply(bp$true_AF_case, function(x) ifelse(x > .5, 1-x, x))
bp$maf_control <- sapply(bp$true_AF_control, function(x) ifelse(x > .5, 1-x, x))

# calculate AC columns
sz$MAC_case <- sz$maf_case*nCase_sz*2
sz$MAC_control <- sz$maf_control*nControl_sz*2
bp$MAC_case <- bp$maf_case*nCase_bp*2
bp$MAC_control <- bp$maf_control*nControl_bp*2

sz <- sz[sz$MAC_case > 40 & sz$MAC_control > 40, ]
bp <- bp[bp$MAC_case > 40 & bp$MAC_control > 40, ]

# setup the dataframes as needed:
# need chromosome and position columns from locus
sz$id <- sz$locus
sz <- sz %>%
  separate(locus, into = c("chr", "pos"), sep = ":")
sz$chr <- sub("chr", "", sz$chr)

bp$id <- bp$locus
bp <- bp %>%
  separate(locus, into = c("chr", "pos"), sep = ":")
bp$chr <- sub("chr", "", bp$chr)

sz[sz$chr == "X", ]$chr <- 23
sz[sz$chr == "Y", ]$chr <- 24
bp[bp$chr == "X", ]$chr <- 23
bp[bp$chr == "Y", ]$chr <- 24

bp$chr <- as.numeric(bp$chr)
bp$pos <- as.numeric(bp$pos)
sz$chr <- as.numeric(sz$chr)
sz$pos <- as.numeric(sz$pos)

# now need allele1 and allele2
sz <- sz %>%
  mutate(alleles = str_extract(alleles, "\\[(.*?)\\]"),
         alleles = str_remove_all(alleles, "[\\[\\]\"\\s]")) %>%
  separate(alleles, into = c("A1", "A2"), sep = ",")
bp <- bp  %>%
  mutate(alleles = str_extract(alleles, "\\[(.*?)\\]"),
         alleles = str_remove_all(alleles, "[\\[\\]\"\\s]")) %>%
  separate(alleles, into = c("A1", "A2"), sep = ",")

sz$RSID <- sz$variant_id
bp$RSID <- bp$variant_id

sz$OR <- exp(sz$BETA)
bp$OR <- exp(bp$BETA)

sz <- as.data.frame(sz)
bp <- as.data.frame(bp)
```

Estimate the case and control AFs using CaseControl_SE and CaseControl_AF
```{r CaseControlAF, eval=F}
# get case and control AF using SE
sz <- cbind(sz, CaseControl_SE(N_case = nCase_sz,
                        N_control = nControl_sz,
                        OR = exp(sz$BETA),
                        SE = sz$SE))
bp <- cbind(bp, CaseControl_SE(N_case = nCase_bp,
                        N_control = nControl_bp,
                        OR = exp(bp$BETA),
                        SE = bp$SE))
sz <- as.data.frame(sz)
bp <- as.data.frame(bp)

# get case and control AF using AF
sz <- cbind(sz, CaseControl_AF(N_case = nCase_sz,
                        N_control = nControl_sz,
                        OR = sz$OR,
                        AF_population = sz$AF_Allele2))
bp <- cbind(bp, CaseControl_AF(N_case = nCase_bp,
                        N_control = nControl_bp,
                        OR = bp$OR,
                        AF_population = bp$AF_Allele2))
sz <- as.data.frame(sz)
bp <- as.data.frame(bp)
```

Let's examine the estimated AFs compared to the true AFs

### CaseControl_AF estimates

```{r estAF_plots, eval=F}
sz$maf <- (sz$maf_case*nCase_sz + sz$maf_control*nControl_sz)/(nCase_sz + nControl_sz)
bp$maf <- (bp$maf_case*nCase_bp + bp$maf_control*nControl_bp)/(nCase_bp + nControl_bp)

breaks <- seq(0.0, 0.5, by = 0.1)
sz$bin <- cut(sz$maf, breaks = breaks, include.lowest = T, right = FALSE)
bp$bin <- cut(bp$maf, breaks = breaks, include.lowest = T, right = FALSE)

AFdata <- data.frame(trueMAF = c(sz$true_AF_case, sz$true_AF_control, bp$true_AF_case, 
                                 bp$true_AF_control),
                     estMAF = c(sz$AF_case, sz$AF_control, bp$AF_case, bp$AF_control),
                     bin = c(sz$bin, sz$bin, bp$bin, bp$bin),
                     trait = c(rep("SZ", nrow(sz)*2), rep("BP", nrow(bp)*2)),
                     status = c(rep("Case", nrow(sz)), rep("Control", nrow(sz)),
                                rep("Case", nrow(bp)), rep("Control", nrow(bp))))
p_AF <- ggplot(AFdata, aes(x = bin, y = (estMAF-trueMAF), fill = trait)) +
  geom_boxplot() + 
  scale_fill_manual(values = c("#495CA0", "goldenrod")) +
  facet_wrap(~status) +
  ggtitle("CaseControl_AF Estimates") +
  theme_bw() +
  ylim(c(-0.075, 0.075)) +
  ylab("Estimated - True")
ggsave(plot = p_AF, filename = "szbp_CaseControl_AF_boxplot.png", width = 14, height = 8, 
       units = "in")
```
![](szbp_CaseControl_AF_boxplot.png)
**Figure 1**. The bias (Estimated MAF - True MAF) for the variants by MAF bin (x-axis) for each
data set estimated using CaseControl_AF. We see that overall the estimates are very accurate (i.e.
the bias is close to 0). For the cases, which have much smaller sample size, there is more 
variability in the estimates. 

### CaseControl_SE estimates

```{r, eval=F}
SEdata <- data.frame(trueMAF = c(sz$maf_case, sz$maf_control, bp$maf_case, bp$maf_control),
                     estMAF = c(sz$MAF_case, sz$MAF_control, bp$MAF_case, bp$MAF_control),
                     bin = c(sz$bin, sz$bin, bp$bin, bp$bin),
                     trait = c(rep("SZ", nrow(sz)*2), rep("BP", nrow(bp)*2)),
                     status = c(rep("Case", nrow(sz)), rep("Control", nrow(sz)),
                                rep("Case", nrow(bp)), rep("Control", nrow(bp))))
p_SE <- ggplot(SEdata, aes(x = bin, y = (estMAF-trueMAF), fill = trait)) +
  geom_boxplot() + 
  scale_fill_manual(values = c("#495CA0", "goldenrod")) +
  facet_wrap(~status) +
  ggtitle("CaseControl_SE Estimates") +
  theme_bw() +
  ylim(c(-0.075, 0.075)) +
  ylab("Estimated - True")
ggsave(plot = p_SE, filename = "szbp_CaseControl_SE_boxplot.png", width = 14, height = 8, 
       units = "in")
```

![](szbp_CaseControl_SE_boxplot.png)
**Figure 2**. The bias (Estimated MAF - True MAF) for the variants by MAF bin (x-axis) for each
data set estimated using CaseControl_SE. We see that the estimated MAF has increasing bias for the
larger MAF bins. This is true of both SZ and BP.

We see that overall results are consistent with those seen in Chapter III simulations and real data.
The CaseControl_AF results are quite accurate (Figure 1), the CaseControl_SE results are biased at 
higher MAFs (Figure 2).We also see the same trend where larger case sample sizes have overall 
smaller SEs, which results in increased bias, which is why the Bipolar ($N_{case} = 2856$) 
CaseControl_SE estimates are more biased than the Schizophrenia ($N_{case} = 584$) CaseControl_SE 
estimates. 

Define sample overlap matrices
```{r overlap_matrices, eval=F}
# a is N is study 1, d is N in study 2, b = c = number overlapping cases
case_in_case <- matrix(data = c(nCase_sz, 0, 0, nCase_bp), nrow = 2)

# a in N control in study 1, d is number of controls in study 2, b = c = overlapping controls
control_in_control <- matrix(data = c(nControl_sz, nControl_bp, nControl_bp, nControl_bp), nrow = 2)

# a = d = 0; b is number cases in trait 1 that are controls for trait 2; c is number of cases from trait 2 that are controls as trait 1
case_in_control <- matrix(data =  c(0, 0, 0, 0), nrow = 2)
```

Run CC-GWAS using different case and control AFs:

1. true known case and control AFs
2. CaseControl_AF estimated case and control AFs
3. CaseControl_SE estimated case and control AFs

### Using known AFs

```{r CCGWAS_knownAF, eval=F}
res_true_af <- CC_GWAS(trait1_data = sz,
                       trait2_data = bp,
                       trait1_N_case = nCase_sz,
                       trait2_N_case = nCase_bp,
                       trait1_N_control = nControl_sz,
                       trait2_N_control = nControl_bp,
                       AF_case_colname = "true_AF_case",
                       AF_control_colname = "true_AF_control",
                       chromosome_colname = "chr",
                       position_colname = "pos",
                       id_colname = "id",
                       A1_colname = "A1",
                       A2_colname = "A2",
                       rsid_colname = "RSID",
                       OR_colname = "OR",
                       SE_colname = "SE",
                       N_caseInCase = case_in_case,
                       N_controlInControl = control_in_control,
                       N_caseInControl = case_in_control,
                       Zthreshold = 1.0)

CC_GWAS_plot(results = res_true_af, title = "SCZ vs BIP All of Us - known AFs")
```

![](scz_bp_aou_trueAF_MACFiltered_scale.png)
**Figure 3**. Manhattan plot of the CC-GWAS results using the known case and control AFs for 
SZ and BP. The red dashed line is the Bonferroni-corrected genome-wide significance threshold. 
Chromosome 23 is the X chromosome and chromosome 24 is the Y chromosome.

### Using CaseControl_AF

```{r ccGWAS_CaseControl_AF, eval=F}
res_af <- CC_GWAS(trait1_data = sz,
                  trait2_data = bp,
                  trait1_N_case = nCase_sz,
                  trait2_N_case = nCase_bp,
                  trait1_N_control = nControl_sz,
                  trait2_N_control = nControl_bp,
                  AF_case_colname = "AF_case",
                  AF_control_colname = "AF_control",
                  chromosome_colname = "chr",
                  position_colname = "pos",
                  id_colname = "id",
                  A1_colname = "A1",
                  A2_colname = "A2",
                  rsid_colname = "RSID",
                  OR_colname = "OR",
                  SE_colname = "SE",
                  N_caseInCase = case_in_case,
                  N_controlInControl = control_in_control,
                  N_caseInControl = case_in_control,
                  Zthreshold = 1.0)

CC_GWAS_plot(results = res_af, title = "SCZ vs BIP All of Us - CaseControl_AF")
```

![](scz_bp_aou_AF_MACFiltered_scale.png)
**Figure 4**. Manhattan plot of the CC-GWAS results using the CaseControl_AF estimated  case and 
control AFs for SZ and BP. The red dashed line is the Bonferroni-corrected genome-wide significance 
threshold. Chromosome 23 is the X chromosome and chromosome 24 is the Y chromosome.

### Using CaseControl_SE

```{r ccGWAS_CaseControl_SE, eval=FALSE}
res_se <- CC_GWAS(trait1_data = sz,
                  trait2_data = bp,
                  trait1_N_case = nCase_sz,
                  trait2_N_case = nCase_bp,
                  trait1_N_control = nControl_sz,
                  trait2_N_control = nControl_bp,
                  AF_case_colname = "MAF_case",
                  AF_control_colname = "MAF_control",
                  chromosome_colname = "chr",
                  position_colname = "pos",
                  id_colname = "id",
                  A1_colname = "A1",
                  A2_colname = "A2",
                  rsid_colname = "RSID",
                  OR_colname = "OR",
                  SE_colname = "SE",
                  N_caseInCase = case_in_case,
                  N_controlInControl = control_in_control,
                  N_caseInControl = case_in_control,
                  Zthreshold = 1.0)

CC_GWAS_plot(results = res_se, title = "SCZ vs BIP All of Us - CaseControl_SE")
```

![](scz_bp_aou_SE_MACFiltered_scale.png)
**Figure 5**. Manhattan plot of the CC-GWAS results using the CaseControl_SE estimated  case and 
control AFs for SZ and BP. The red dashed line is the Bonferroni-corrected genome-wide significance 
threshold. Chromosome 23 is the X chromosome and chromosome 24 is the Y chromosome.

QQ-Plot

![](qq_szbp_all_new_lambda.png)
**Figure 6**. Q-Q Plot for the All of Us Schizophrenia vs Bipolar CC-GWAS. The results from using
the known AFs are shown in green, the results from using CaseControl_AF estimated AFs are shown 
in black, and the results from using CaseControl_SE estimated AFs are shown in purple.

We see that the CaseControl_SE derived MAFs result in slightly deflated test statistics. 
Conversely, CaseControl_AF derived AFs results in inflated test statistics. As expected, the true
AFs do not appear to be inflated. 

## Replicating the resuts from Yang et al

As a sanity check, we ensured that we could replicate the results from the ReACt paper which are
shown below. In the paper they used the summary statistics from the [Psychiatric Genomics Consortium 
(PGC)](https://www.med.unc.edu/pgc/download-results/). We are not able to compare the three AFs 
(known case and control AFs, CaseControl_AF estimated, CaseControl_SE estimated) for this dataset as
they d0 not include total AF. However, to ensure that no errors have been made in adapting the code
from C to R, we ensured that we could obtain the same results as published in the ReACt paper using
the code presented here. This is using CaseControl_SE to estimate the case and control AFs, and 
subsequently performing CC-GWAS. These results are nearly identical to those published in the paper.

![](manahttan_ReACtrep_szbp.png)

---
title: "CCAFE Extra Details"
author: "Hayley Stoneman"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
    toc_depth: 2
vignette: >
  %\VignetteIndexEntry{CCAFE Extra Details}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)
```

# Introduction

In this vignette we will share a few extra details regarding the usage of the functions in 
CCAFE. 

For the vignette with basic usage and examples see the CCAFE Vignette. 

## Simulated dataset

Will create a simulated dataset to explore the issues discussed in this vignette. 

This dataset will have:

* 2400 variants (100 on each chromosome)
* 10000 cases and 10000 controls
  * 5000 XX cases and 5000 XY cases
  * 5000 XX controls and 5000 XY controls

```{r, echo=TRUE, message=FALSE}
library(tidyverse)
library(CCAFE)
library(DescTools)
library(cowplot)
```


```{r Simulations}
# need to create simulated AFs from autosomes and sex chromosomes

# simulate 100 variants from each chromosome

# will simulate from a sample of 5000 XX and 5000 XY case individuals
# then add 5000 of each in control

set.seed(2020)

N_XX <- 5000
N_XY <- 5000
N <- N_XX + N_XY

for(chr in 1:24) {
  refs <- runif(n = 100, min = 0, max = 1)
  if(chr < 23) {
    genos <- t(sapply(refs, function(x) {
      rmultinom(1, N, c(x^2, 2*x*(1-x), (1-x)^2))
    }))
    afs_toadd <- apply(genos, 1, function(x) ((x[1]*2)+x[2])/(2*N))
  } else {
    if(chr == 23) { # simulate X chromosome variants
      XX_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XX*2, 1, x))
      }))
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      total_alleles <- XX_alleles + XY_alleles
      afs_toadd <- total_alleles/(2*N_XX + N_XY)
    } else { # simulate Y chromosome variants
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      afs_toadd <- XY_alleles/(N_XY)
    }
  }
  if(chr == 1) {
    simDat <- data.frame(chr = chr, pos = c(1:100), case_sim_af = afs_toadd)
  } else {
    simDat <- rbind(simDat, data.frame(chr = chr, pos = c(1:100), case_sim_af = afs_toadd))
  }
}

simDat$control_sim_af <- 0

# simulate control AFs

for(chr in 1:24) {
  refs <- simDat[simDat$chr == chr,]$case_sim_af
  refs <- refs - runif(n = length(refs), min = 0, max = .025)
  refs[refs < 0] <- 0
  refs[refs > 1] <- 1
  
  if(chr < 23) {
    genos <- t(sapply(refs, function(x) {
      rmultinom(1, N, c(x^2, 2*x*(1-x), (1-x)^2))
    }))
    afs_toadd <- apply(genos, 1, function(x) ((x[1]*2)+x[2])/(2*N))
  } else {
    if(chr == 23) { # simulate X chromosome variants
      XX_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XX*2, 1, x))
      }))
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      total_alleles <- XX_alleles + XY_alleles
      afs_toadd <- total_alleles/(2*N_XX + N_XY)
    } else { # simulate Y chromosome variants
      XY_alleles <- (sapply(refs, function(x) {
        sum(rbinom(N_XY, 1, x))
      }))
      
      afs_toadd <- XY_alleles/(N_XY)
    }
  }
  simDat[simDat$chr == chr, ]$control_sim_af <- afs_toadd
}

simDat$total_sim_af <- (simDat$case_sim_af*10000 + simDat$control_sim_af*10000)/(10000 + 10000)

# need to add OR and SE
OR <- rep(0, nrow(simDat))
SE <- rep(0, nrow(simDat))

for(i in 1:nrow(simDat)) {
  AF1 = simDat[i, ]$case_sim_af
  AF2 = simDat[i,]$control_sim_af
  if(simDat[i,]$chr < 23) {
    # first calculate the 2x2 tables of allele counts
    a = AF1 * 2 * 10000 
    b = (1-AF1) * 2 * 10000
    c = AF2 * 2 * 10000
    d = (1-AF2) * 2 * 10000
  } else {
    if(simDat[i,]$chr == 23) {
      a = AF1 * (2*5000 + 5000)
      b = (1-AF1) * (2*5000 + 5000)
      c = AF2 * (2*5000 + 5000)
      d = (1-AF2) * (2*5000 + 5000)
    } else {
      a = AF1 * 5000
      b = (1-AF1) * 5000
      c = AF2 * 5000
      d = (1-AF2) * 5000
    }
  }
  
  OR[i] <- (a*d)/(b*c)
  SE[i] <- sqrt(1/a + 1/b + 1/c + 1/d)
}

simDat$OR <- OR
simDat$SE <- SE

simDat[simDat$chr == 23, ]$chr <- "X"
simDat[simDat$chr == 24, ]$chr <- "Y"

simDat <- simDat %>% rowwise() %>%
  mutate(is_minor_case = ifelse(case_sim_af <= 0.5, 1, 0),
         is_minor_control = ifelse(control_sim_af <= 0.5, 1, 0),
         is_minor_total = ifelse(total_sim_af <= 0.5, 1, 0)) %>%
  mutate(case_sim_maf = ifelse(is_minor_case, case_sim_af, 1-case_sim_af),
         control_sim_maf = ifelse(is_minor_control, control_sim_af, 1-control_sim_af),
         total_sim_maf = ifelse(is_minor_total, total_sim_af, 1-total_sim_af))

head(simDat)
```

# MAF vs AF

## Minor alleles in CaseControl_SE

A key assumption made by the implementation of CaseControl_SE is that the frequency being estimated
is that of the *minor* allele. As such, the returned value is always the minor allele and bounded
[0,0.5], regardless of which allele was used for calculation of the input OR and SE. This can 
create complications in data harmonization between multiple datasets, because the minor allele may
not always be the same between the two datasets. We note that care should be taken when using this
method to reconstruct case and control AFs for use in secondary analyses as the alleles being 
compared may not be retained. 

### Examples in simulated data

```{r Any mismatched alleles}
nrow(simDat[simDat$is_minor_case != simDat$is_minor_control, ])
```
There are 32 variants for which the minor allele is a different allele in cases and controls. 

```{r Show mismatched alleles}
diff_alleles <- simDat[simDat$is_minor_case != simDat$is_minor_control, ]
diff_alleles
```

This will results in different alleles being reported from CaseControl_SE, which, if used for
subsequent analyses like case case GWAS (CC-GWAS) would result in the user comparing different 
alleles between datasets/groups. 

## Using total minor allele frequency in CaseControl_AF

In analyses for the manuscript we assessed using the total sample MAF compared to the total sample
AF. Here we noted that converting the total AF to MAF first (and using that as input) introduces 
variability due to rounding. 

### Demonstration in simulations

Here we will show the effects of converting to MAF in 3 scenarios:

1. No AF conversions for calculations, No conversions for comparison
2. No AF conversions for calculations, convert to MAF for comparison
3. Convert to MAF for calculations, covert to MAF for comparison

```{r Calculate AFs}
simDat <- simDat[is.finite(simDat$OR), ]
simDat <- as.data.frame(simDat)

res_AF <- CaseControl_AF(data = simDat,
                         N_case = 10000,
                         N_control = 10000,
                         OR_colname = "OR",
                         AF_total_colname = "total_sim_af")

res_MAF <- CaseControl_AF(data = simDat,
                         N_case = 10000,
                         N_control = 10000,
                         OR_colname = "OR",
                         AF_total_colname = "total_sim_maf")
```

```{r Plot_AF_results, message=FALSE}
p_1 <- ggplot(res_AF, aes(x = case_sim_af, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True AF") +
  ylab("Estimated AF") +
  geom_text(x = .175, y = .9, label = paste0("CCC=", round(DescTools::CCC(res_AF$case_sim_af, res_AF$AF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("1. No Conversion")

res_AF$MAF_case <- sapply(res_AF$AF_case, function(x) ifelse(x > 0.5, 1-x, x))
res_AF <- as.data.frame(res_AF)
p_2 <- ggplot(res_AF, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  geom_text(x = .1, y = .45, label = paste0("CCC=", round(DescTools::CCC(res_AF$case_sim_maf, res_AF$MAF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("2. Convert AFTER")

p_3 <- ggplot(res_MAF, aes(x = case_sim_maf, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  geom_text(x = .1, y = .48, label = paste0("CCC=", round(DescTools::CCC(res_MAF$case_sim_maf, res_MAF$AF_case, na.rm = TRUE)$rho.c$est, 4))) +
  ggtitle("3. Convert BEFORE")

cowplot::plot_grid(p_1, p_2, p_3, ncol = 3)
```
Here it's clear to see that as long as the user does not convert to MAF before estimating the
case and control AFs, then we get perfect reconstruction of the AFs. Conversely, converting the 
total sample AF to MAF first introduces variability around the estimates (seen in scenario 3). 
Overall, the estimates are still highly accurate, but when possible we recommend not converting
AFs to MAF before inputting to CaseControl_AF.

# Variants on sex chromosomes

Another potential problem that users may encounter is the presence of sex chromosomes when using
CaseControl_SE. 

The derivation for the framework underlying CaseControl_SE utilizes the total allele number (AN) 
for cases and controls. The original proposed method in ReACt used the equation AN = 2*N, where N
is the sample size. However, this is only true in all samples for autosomes. This equation will
also hold true for the X chromosome in an all biological female (XX) sample. However, in any sample
in which there are both XX and XY individuals, this equation will not give the correct total AN
for the sex chromosomes. 

We have added the ability to estimate the case and control MAF for the sex chromosomes by employing
the following equations for the AN in the original framework:

$$ AN_X = 2*N_{XX individuals} + N_{XY individuals} $$
$$ AN_Y = N_{XY individuals} $$
$$ AN_{autosomes} = 2*N$$
Failing to use the correct AN resulted in MAF estimates that did not follow the same trend as the
autosomes. We require chromosome data with the input for CaseControl_SE to check for the presence of
variants on sex chromosomes. Notably, in order to estimate the MAFs for these sex chromosomes, users
must know and provide as input the number of XX and XY individuals per case and control sample. 
While it is reasonable to expect most studies to publish this demographic information, it may not
always be available. In this case, users can set the flag *'remove_sex_chromosomes'* to TRUE and
the method will return the MAFs for only autosomal variants. We will demonstrate below what the 
results may look like if the sex chromosomes are not properly accounted for. 

The implementation of CaseControl_AF directly uses sample size and total AF, rather than total AN. 
Thus this method is not sensitive to the XX and XY specific samples sizes. We will also show this 
below. 

### Demonstration in simulations

```{r ceate unaccounted for sex chr data}
# to simulate a dataset in which the sex chromosomes are not properly accounted for, we will falsely
# rename the x and y chromosomes to autosomes
simDat_nosex <- simDat
simDat_nosex[simDat_nosex$chr == "X", ]$chr <- 1
simDat_nosex[simDat_nosex$chr == "Y", ]$chr <- 2
```

```{r Get estimated AF/MAF}
af_res_sex <- CaseControl_AF(data = simDat,
                           N_case = 10000,
                           N_control = 10000,
                           OR_colname = "OR",
                           AF_total_colname = "total_sim_af")

se_res <- CaseControl_SE(data = simDat_nosex,
                                N_case = 10000,
                                N_control = 10000,
                                OR_colname = "OR",
                                SE_colname = "SE",
                                chromosome_colname = "chr",
                                position_colname = "pos",
                                sex_chromosomes = FALSE,
                                do_correction = FALSE)

se_res_sex <- CaseControl_SE(data = simDat,
                             N_case = 10000,
                             N_control = 10000,
                             OR_colname = "OR",
                             SE_colname = "SE",
                             chromosome_colname = "chr",
                             position_colname = "pos",
                             sex_chromosomes = TRUE,
                             remove_sex_chromosomes = FALSE,
                             do_correction = FALSE,
                             N_XX_case = 5000,
                             N_XX_control = 5000,
                             N_XY_case = 5000,
                             N_XY_control = 5000)
```

Examine the following scenarios:

1. CaseControl_AF with sex chromosomes
2. CaseControl_SE with sex chromosomes improperly accounted for
3. CaseControl_SE with sex chromosomes properly accounted for

```{r plot_sex_chromosome_results, message=FALSE}
p_a <- ggplot(af_res_sex, aes(x = case_sim_af, y = AF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True AF") +
  ylab("Estimated AF") +
  ggtitle("1. CaseControl_AF")

p_b <- ggplot(se_res, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  ggtitle("2. CaseControl_SE") 

p_c <- ggplot(se_res_sex, aes(x = case_sim_maf, y = MAF_case)) +
  geom_point() +
  geom_abline(color = "red", linetype = "dashed") +
  theme_bw() +
  xlab("True MAF") +
  ylab("Estimated MAF") +
  ggtitle("3. CaseControl_SE") 

plot_grid(p_a, p_b, p_c, ncol =3)
```

# Session Info
```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CCAFE_convertVCF.R
\name{CCAFE_convertVCF}
\alias{CCAFE_convertVCF}
\title{CCAFE_convertVCF}
\usage{
CCAFE_convertVCF(vcf)
}
\arguments{
\item{vcf}{a Variant Call Format (VCF) file read in using VariantAnnotation
BioConductor package}
}
\value{
a dataframe object with columns Position, RSID, Chromosome, REF,
ALT, beta, SE, AF, OR
}
\description{
Formats information from a VCF object for use in CCAFE methods as follows:
From the rowRanges object: seqnames (chromosome), ranges (position),
From the geno object: ES (effect size of ALT), SE, AF (allele frequency
of ALT)
}
\examples{
library(VariantAnnotation)
library(CCAFE)

# load the data
data("vcf_sample")

# run the method
df_sample <- CCAFE_convertVCF(vcf_sample)
print(head(df_sample))

# can then use in CCAFE methods
# since we have total AF, will use CaseControl_AF
df_sample <- CaseControl_AF(data = df_sample,
                          N_case = 48286,
                          N_control = 250671,
                          OR_colname = "OR",
                          AF_total_colname = "AF")
head(df_sample)
}
\author{
Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcf_sample.R
\docType{data}
\name{vcf_sample}
\alias{vcf_sample}
\title{A VCF from this GWAS of Type 2 Diabetes https://doi.org/10.1038/s41588-018-0084-1. containing
a subset of 10,000 variants}
\format{
`vcf_sample`
A CollapsedVCF

dim: 10000 1
rowRanges(vcf):
  GRanges with 5 metadata columns: paramRangeID, REF, ALT, QUAL, FILTER
info(vcf):
  DataFrame with 1 column: AF
info(header(vcf)):
  Number Type  Description
AF A      Float Allele Frequency
geno(vcf):
  List of length 9: ES, SE, LP, AF, SS, EZ, SI, NC, ID
geno(header(vcf)):
  Number Type    Description
ES A      Float   Effect size estimate relative to the alternative allele
SE A      Float   Standard error of effect size estimate
LP A      Float   -log10 p-value for effect estimate
AF A      Float   Alternate allele frequency in the association study
SS A      Integer Sample size used to estimate genetic effect
EZ A      Float   Z-score provided if it was used to derive the EFFECT and SE fields
SI A      Float   Accuracy score of summary data imputation
NC A      Integer Number of cases used to estimate genetic effect
ID 1      String  Study variant identifier
}
\usage{
data("vcf_sample")
}
\description{
A VCF from this GWAS of Type 2 Diabetes https://doi.org/10.1038/s41588-018-0084-1. containing
a subset of 10,000 variants
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CaseControl_AF.R
\name{CaseControl_AF}
\alias{CaseControl_AF}
\title{CaseControl_AF}
\usage{
CaseControl_AF(
  data,
  N_case = 0,
  N_control = 0,
  OR_colname = "OR",
  AF_total_colname = "AF"
)
}
\arguments{
\item{data}{dataframe with each row being a variant and columns for
AF_total and OR}

\item{N_case}{the number of cases in the sample}

\item{N_control}{the number of controls in the sample}

\item{OR_colname}{a string containing the exact column name in 'data'
with the OR}

\item{AF_total_colname}{a string containing the exact column name in 'data'
with the whole sample AF}
}
\value{
returns a dataframe with two columns (AF_case, AF_control) and rows
equal to the number of variants
}
\description{
This is a function to derive the case and control AFs from GWAS summary
statistics when the user has access to the whole sample AF, the sample sizes,
and the OR (or beta).
If user has SE instead of sample AF use \code{\link[=CaseControl_SE]{CaseControl_SE()}}
}
\examples{
library(CCAFE)

data("sampleDat")
sampleDat <- as.data.frame(sampleDat)

nCase_sample = 16550
nControl_sample = 403923

# get the estimated case and control AFs
af_method_results <- CaseControl_AF(data = sampleDat,
                                    N_case = nCase_sample,
                                    N_control = nControl_sample,
                                    OR_colname = "OR",
                                    AF_total_colname = "true_maf_pop")

head(af_method_results)

}
\references{
https://github.com/wolffha/CCAFE
}
\seealso{
\url{https://github.com/wolffha/CCAFE} for further documentation
}
\author{
Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CCAFE-package.R
\docType{package}
\name{CCAFE-package}
\alias{CCAFE-package}
\title{CCAFE: Case Control Allele Frequency Estimation}
\description{
Functions to reconstruct case and control AFs from summary statistics. One function uses OR, NCase, NControl, and SE(log(OR)). The second function uses OR, NCase, NControl, and AF for the whole sample.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/wolffha/CCAFE/}
  \item Report bugs at \url{https://github.com/wolffha/CCAFE/issues}
}

}
\author{
\strong{Maintainer}: Hayley Wolff \email{hayley.wolff@cuanschutz.edu}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CaseControl_SE.R
\name{CaseControl_SE}
\alias{CaseControl_SE}
\title{CaseControl_SE}
\usage{
CaseControl_SE(
  data,
  N_case = 0,
  N_control = 0,
  OR_colname = "OR",
  SE_colname = "SE",
  chromosome_colname = "chr",
  sex_chromosomes = FALSE,
  position_colname = "pos",
  N_XX_case = NA,
  N_XX_control = NA,
  N_XY_case = NA,
  N_XY_control = NA,
  do_correction = FALSE,
  correction_data = NA,
  remove_sex_chromosomes = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{data}{dataframe where each row is a variant and columns contain the
OR, SE, chromosome and positions}

\item{N_case}{an integer of the number of Case individuals}

\item{N_control}{an integer of the number of Control individuals}

\item{OR_colname}{a string containing the exact column name in 'data' with
the OR}

\item{SE_colname}{a string containing the exact column name in 'data' with
the SE}

\item{chromosome_colname}{a string containing the exact column name in 'data'
with the chromosomes, default "chr"}

\item{sex_chromosomes}{boolean, TRUE if variants from sex chromosomes are
included in the dataset. Sex chromosomes can be numeric (23, 24) or
character (X, Y). If numeric, assumes X=23 and Y=24.}

\item{position_colname}{a string containing the exact column name in 'data'
with the position, default "pos"}

\item{N_XX_case}{the number of XX chromosome case individuals
(REQUIRED if sex_chromosomes == TRUE)}

\item{N_XX_control}{the number of XX chromosome control individuals
(REQUIRED if sex_chromosomes == TRUE)}

\item{N_XY_case}{the number of XY chromosome case individuals
(REQUIRED if sex_chromosomes == TRUE)}

\item{N_XY_control}{the number of XY chromosome control individuals
(REQUIRED if sex_chromosomes == TRUE)}

\item{do_correction}{boolean, TRUE if data is provided to perform correction}

\item{correction_data}{a dataframe with the following exact columns: CHR,
POS, proxy_MAF with data that is harmonized between the proxy true datasets
and the observed dataset}

\item{remove_sex_chromosomes}{boolean, TRUE if should keep autosomes only.
This is needed when the number of biological sex males/females per case and
control group is not known.}

\item{verbose}{boolean, determine whether warnings should be displayed
(default FALSE)}
}
\value{
returns data as a dataframe with three additional columns: MAF_case,
MAF_control, MAF_total for the estimated MAFs for each variant. If
do_correction = TRUE, then will output 3 additional columns
(MAF_case_adj, MAF_control_adj, MAF_total_adj) with the adjusted estimates.
}
\description{
This is a function to derive the case, control, and total MAFs
from GWAS summary statistics when the user has access to the sample sizes,
and the OR (or beta), and SE for the log(OR) for each variant.
If user has total AF instead of SE use \code{\link[=CaseControl_AF]{CaseControl_AF()}}
This code uses the GroupFreq function adapted from C from
\url{https://github.com/Paschou-Lab/ReAct/blob/main/GrpPRS_src/CountConstruct.c}
}
\examples{
library(CCAFE)

data("sampleDat")
sampleDat <- as.data.frame(sampleDat)

nCase_sample = 16550
nControl_sample = 403923

# get the estimated case and control MAFs
se_method_results <- CaseControl_SE(data = sampleDat,
                                    N_case = nCase_sample,
                                    N_control = nControl_sample,
                                    OR_colname = "OR",
                                    SE_colname = "SE",
                                    chromosome_colname = "CHR",
                                    position_colname = "POS")

head(se_method_results)

}
\references{
https://github.com/wolffha/CCAFE
}
\seealso{
\url{https://github.com/wolffha/CCAFE} for further documentation
}
\author{
Hayley Wolff (Stoneman), \email{hayley.wolff@cuanschutz.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleDat.R
\docType{data}
\name{sampleDat}
\alias{sampleDat}
\title{PanUKBB and gnomAD Diabetes Data}
\format{
`sampleDat`
A data frame with 500 rows and 11 columns:
\describe{
  \item{CHR}{chromosome number}
  \item{POS}{base-pair position of variant (GRCh37 coordinates)}
  \item{REF}{Reference allele}
  \item{ALT}{Alternate allele}
  \item{true_maf_case}{MAF in EUR cases in Pan-UKBB Diabetes}
  \item{true_maf_control}{MAF in EUR controls in Pan-UKBB Diabetes}
  \item{true_maf_pop}{MAF in whole EUR sample in Pan-UKBB Diabetes}
  \item{beta}{beta from EUR GWAS in Pan-UKBB Diabetes}
  \item{SE}{SE of beta from EUR GWAS in Pan-UKBB Diabetes}
  \item{OR}{OR from EUR GWAS in Pan-UKBB Diabetes}
  \item{gnomad_maf}{MAF in gnomAD NFE}
}
}
\source{
<https://pan.ukbb.broadinstitute.org/docs/per-phenotype-files>

<https://gnomad.broadinstitute.org/downloads>
}
\usage{
data("sampleDat")
}
\description{
This is a subset of 500 variants on chromosome 1 from the Pan-UKBB diabetes GWAS with the whole sample (pop), case, and control minor allele frequency (MAF) for those classified in Pan-UKBB as European (EUR).
These variants (which are mapped to GRCh37) have been harmonized with gnomAD non-Finnish European (NFE) MAFs.
}
\keyword{datasets}
